package gssim.schedframe.scheduling;


import gridsim.Gridlet;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.joda.time.DateTime;
import org.joda.time.DateTimeUtilsExt;
import org.joda.time.ReadableDuration;

import schedframe.scheduling.WorkloadUnitHandler;
import schedframe.scheduling.manager.tasks.JobRegistryImpl;
import schedframe.scheduling.policy.AbstractManagementSystem;
import schedframe.scheduling.tasks.AbstractProcesses;
import schedframe.scheduling.tasks.AbstractProcessesGroup;
import schedframe.scheduling.tasks.Task;
import schedframe.scheduling.tasks.TaskInterface;
import schedframe.scheduling.tasks.requirements.ResourceParameterName;

/**
 * 
 * @author Marcin Krystek
 *
 */
public class Executable extends Gridlet implements ExecTask{

	protected Task task;
	protected String processesSetId;
	
	protected List<String> visitedResources;
	

	
	public Executable(Task t){
		super(0, 0, 0, 0);
		this.task = t;
		super.gridletID_ = (t.getJobId() + "_" + t.getId()).hashCode();
		reservationId = null;
		this.visitedResources = new ArrayList<String>();
		setUserID(task.getSenderId());
	}
	
	public Executable(Task t, AbstractProcesses procesesSet){
		super(0, 0, 0, 0);
		this.task = t;
		this.processesSetId = procesesSet.getId(); 
		super.gridletID_ = (t.getJobId() + "_" + t.getId() + "_" + processesSetId).hashCode();
		this.visitedResources = new ArrayList<String>();
		setUserID(task.getSenderId());
	}
	
	public DateTime getExecutionEndTime() throws NoSuchFieldException {
		return task.getExecutionEndTime();
	}

	public DateTime getExecutionStartTime() throws NoSuchFieldException {
		return task.getExecutionStartTime();
	}

	public ReadableDuration getExpectedDuration() throws NoSuchFieldException {
		return task.getExpectedDuration();
	}

	public String getId() {
		if(processesSetId == null)
			return task.getId();
		else 
			return task.getId() + "_" + processesSetId;
	}

	public String getJobId() {
		return task.getJobId();
	}

	public double getParameterDoubleValue(ResourceParameterName parameterName)
			throws NoSuchFieldException, IllegalArgumentException {
		return task.getParameterDoubleValue(parameterName);
	}

	public String getParameterStringValue(ResourceParameterName parameterName)
			throws NoSuchFieldException, IllegalArgumentException {
		return task.getParameterStringValue(parameterName);
	}

	public DateTime getSubmissionTimeToBroker() {
		return task.getSubmissionTimeToBroker();
	}

	public String getUserDn() {
		return task.getUserDn();
	}

	public List<AbstractProcesses> getProcesses() {
		return task.getProcesses();
	}

	public List<AbstractProcesses> getProcesses(
			AbstractProcessesGroup processGroup) {
		return task.getProcesses(processGroup);
	}

	public List<AbstractProcessesGroup> getProcessesGroups() {
		return task.getProcessesGroups();
	}

	public org.qcg.broker.schemas.resreqs.Task getDescription() {
		return task.getDescription();
	}

	public String getDocument() throws Exception {
		return task.getDocument();
	}
	
	public void setReservationId(String reservationId){
		this.reservationId = reservationId;
	}
	
	public String getProcessesId(){
		return this.processesSetId;
	}
	
	public String getTaskId(){
		return this.task.getId();
	}
	
	public void addSpecificResource(ResourceParameterName resourceName, Object value){
		if(this.chosenResources == null)
			this.chosenResources = new HashMap<ResourceParameterName, Object>();
		
		this.chosenResources.put(resourceName, value);
	}

	public long getWorkloadLogWaitTime() {
		return task.getWorkloadLogWaitTime();
	}

	@Override
	public List<TaskInterface<?>> getTask() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public TaskInterface<?> getTask(String taskId) throws NoSuchFieldException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public int getTaskCount() {
		// TODO Auto-generated method stub
		return 0;
	}
	
	
	/*public void addToResPath(String resName){
		task.addToResPath(resName);
	}
	
	public String getResPath(){
		return task.getResPath();
	}
	*/
	
	public void visitResource(String resName){
		visitedResources.add(resName);
	}
	
	public List<String> getVisitedResources(){
		return visitedResources;
	}
	
	
	protected String reservationId;
	
	public boolean requireReservation(){
		return (reservationId != null);
	}
	
	public String getReservationId(){
		return this.reservationId;
	}
	
	
	public boolean equals(Object obj){
		if(obj instanceof Executable){
			Executable t = (Executable) obj;
			return getId().equals(t.getId());
		}
		return false;
	}
	
	public void setGridletStatus(int newStatus) throws Exception {
		if(status_ == newStatus)
			return;
		
		super.setGridletStatus(newStatus);
		
		if (newStatus == Gridlet.SUCCESS || newStatus == Gridlet.CANCELED) {
            finishTime_ = DateTimeUtilsExt.currentTimeMillis() / 1000;
        }
	}

	/**
	 * @return the sum of the actual CPU time values for every gird resource, 
	 * that this sub task has been executed on
	 */
	public double getTotalProcessingTime() {
		double totalProcessingTime = 0.0;
		for (Object resObj : resList_) {
			Resource res = (Resource) resObj;
			totalProcessingTime += res.actualCPUTime;
		}
		return totalProcessingTime;
	}
	

	protected Map<ResourceParameterName, Object> chosenResources;

	
	/**
	 * 
	 * @return length measured in instructions.
	 */
	public long getLength() {
		return Double.valueOf(getGridletLength()).longValue();
	}
	
	/**
	 * 
	 * @param length measured in instructions.
	 */
	public void setLength(long length){
		setGridletLength(Long.valueOf(length).doubleValue());
	}
	
	/**
	 * 
	 * @return constant which represent current task status
	 */
	public int getStatus() {
		return getGridletStatus();
	}
	
	/**
	 * 
	 * @param status constant which describes current new task status
	 */
	public void setStatus(int status) throws Exception {
		setGridletStatus(status);
	}
	
	public double getCpuCntRequest() throws NoSuchFieldException{
		return getParameterDoubleValue(ResourceParameterName.CPUCOUNT);
	}
	
	public double getMemoryRequest() throws NoSuchFieldException{
		return getParameterDoubleValue(ResourceParameterName.MEMORY);
	}
	
	public boolean isFinished()
    {
        if (index_ == -1) {
            return false;
        }

        boolean completed = false;

        // if result is 0 or -ve then this Gridlet has finished
        double finish = ( (Resource) resList_.get(index_) ).finishedSoFar;
        double result = gridletLength_ - finish;
        if (result <= 0.001) {
            completed = true;
        }

        return completed;
    }
	
	public boolean expectSpecificResource(ResourceParameterName resourceName){
		if(this.chosenResources == null)
			return false;

		return this.chosenResources.containsKey(resourceName);
	}
	
	public Object getExpectedSpecificResource(ResourceParameterName resourceName){
		if(this.chosenResources == null)
			return null;
		
		return this.chosenResources.get(resourceName);
	}
	
	public boolean isRegistered() {
		return task.isRegistered();
	}

	public void register(JobRegistryImpl jobRegistry) {
		task.register(jobRegistry);

	}
	
	@Override
	public void accept(WorkloadUnitHandler wuh) {
		wuh.handleExecutable(this);
	}
}
