package schedframe.resources.computing.profiles.energy.power.ui;

import schedframe.resources.ResourceStatus;
import schedframe.resources.computing.ComputingNode;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.profiles.energy.EnergyEvent;
import schedframe.resources.computing.profiles.energy.EnergyEventType;
import schedframe.resources.computing.profiles.energy.power.PowerProfile;
import schedframe.resources.computing.profiles.energy.power.PowerStateName;
import schedframe.resources.computing.profiles.energy.power.StandardPowerStateName;

public class ComputingNodePowerInterface extends AbstractPowerInterface{

	public static long START_TIME = 600000;
	public static long SHUTDOWN_TIME = 300000;
	public static double START_COST = 4000;
	public static double SHUTDOWN_COST = 2000;

	
	public ComputingNodePowerInterface(ComputingResource resource, PowerProfile pp){
		super(resource, pp);
		currentPowerState = StandardPowerStateName.ON;
	}

	public boolean setPowerState(PowerStateName state){
		if(!supportPowerState(state))
			return false;
		currentPowerState = state;
		ComputingNode computingNode = (ComputingNode) resource;
		if(computingNode.getProcessors() != null)
		{
			for(ComputingResource child:computingNode.getProcessors()){
				child.getPowerInterface().setPowerState(state);
			}
		} else {
			computingNode.handleEvent(new EnergyEvent(EnergyEventType.POWER_STATE_CHANGED, computingNode.getName()));
		}
		
		if(state == StandardPowerStateName.OFF){
			computingNode.setStatus(ResourceStatus.UNAVAILABLE);
		}
		else if(state == StandardPowerStateName.ON){
			computingNode.setStatus(ResourceStatus.FREE);
		}
		//computingNode.handleEvent(new EnergyEvent(EnergyEventType.POWER_STATE_CHANGED, computingNode.getName()));
		return true;
	}
	
	/*public double getPowerConsumption(PowerState state) {
		if(currentPowerState == PowerState.OFF)
			return 0;
		else if(currentPowerState == PowerState.ON)
			return 750;
		else return 500;
	}*/

	/*public boolean supportPowerState(PowerState state) {
		switch(state){
			case ON: 
				return true;
			case OFF: 
				return true;
			case SLEEP: 
				return true;
			case HIBERNATE: 
				return true;
			default: 
				return false;
		}
	}

	public List<PowerState> getSupportedPowerStates() {
		return Arrays.asList(new PowerState[]{PowerState.ON, PowerState.OFF, PowerState.SLEEP, PowerState.HIBERNATE});
	}*/

	public void turnOn(){
		setPowerState(StandardPowerStateName.ON);
	}
	
	public void turnOff(){
		setPowerState(StandardPowerStateName.OFF);
	}



}