package schedframe.scheduling.plugin.grid;

import java.util.List;

import schedframe.scheduling.tasks.JobInterface;
import schedframe.scheduling.tasks.TaskInterface;


/**
 * The JobRegistry interface provides information about jobs and tasks 
 * @author Ariel
 *
 */
public interface JobRegistry extends Module{

	/**
	 * Provide information about submitted jobs which are active (not finished) 
	 * @return array of Job objects 
	 */
	public List<JobInterface<?>> getActiveJobs();

	/**
	 * Provide information about submitted tasks which are active (not finished) 
	 * @return array of Task objects 
	 */
	public List<TaskInterface<?>> getActiveTasks();
	
	/**
	 * Gets information about a given job
	 * @param jobID the identifier of the job
	 * @return object implementing Job interface
	 */
	public JobInterface<?> getJobInfo(String jobID);

	/**
	 * Gets information about a given task
	 * @param jobID the identifier of the job the task belongs to
	 * @param taskId the identifier of the task
	 * @return object implementing {@link TaskInterface} interface
	 */
	public TaskInterface<?> getTaskInfo(String jobID, String taskId);

	/**
	 * @param jobID the identifier of the job
	 * @param job
	 * @return
	 */
	public boolean addJob(JobInterface<?> job);
	
	
	public boolean addTask(TaskInterface<?> task);
	
}
