package org.qcg.broker.schemas.hostparams.wrapper;

import org.qcg.broker.schemas.exception.UnknownParameter;
import org.qcg.broker.schemas.hostparams.ComputingResource;
import org.qcg.broker.schemas.hostparams.HostParameters;
import org.qcg.broker.schemas.hostparams.QueueResource;

/**
 * 
 * @author Marcin Krystek
 *
 */
public interface HostParametersWrapper {

	
	public void wrap(HostParameters hostParams);
	
	/**
	 * 
	 * @return list of all queue resources. If no queue resources can be found, it should
	 * return null. 
	 * @throws UnknownParameter
	 */
	public QueueResource[] getQueueResources() throws UnknownParameter;
	
	/**
	 * 
	 * @return list of all computing resources. If no computing resources can be found, it should
	 * return null.
	 * @throws UnknownParameter
	 */
	public ComputingResource[] getComputingResources() throws UnknownParameter;
	
	/**
	 * 
	 * @return list of standalone resources - the resources which have no reference to any frontend or queue resource.
	 * If there are no such resources, it should return null. 
	 * @throws UnknownParameter
	 */
	public ComputingResource[] getStandaloneComputingResources() throws UnknownParameter;
	
	/**
	 * 
	 * @param id queue id.
	 * @return QueueResource of specified id.
	 * @throws UnknownParameter
	 */
	public QueueResource getQueueResource(String id) throws UnknownParameter;
	
	/**
	 * 
	 * @param id computing resource id.
	 * @return ComputingResource of specified id.
	 * @throws UnknownParameter
	 */
	public ComputingResource getComputingResource(String id) throws UnknownParameter;
	
	/**
	 * 
	 * @return list of computing resources which are frontends to more complex systems (eg. queuing system)
	 * @throws UnknownParameter
	 */
	public ComputingResource[] getFrontends() throws UnknownParameter;
	
}
