package org.qcg.broker.schemas.hostparams.wrapper;

import org.qcg.broker.schemas.exception.NoSuchParamException;
import org.qcg.broker.schemas.hostparams.QueueParameterType;
import org.qcg.broker.schemas.hostparams.QueueResource;
import org.qcg.broker.schemas.hostparams.types.QueueParameterName;

/**
 * 
 * @author Marcin Krystek
 *
 */
public interface QueueResourceParamExplorer {

	/**
	 * Use this method to make {@link QueueResource} object available for exploration.
	 * @param queueResource object that will be explored.
	 */
	public void explore(QueueResource queueResource);
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.QUEUENAME parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getQueueName() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.PRIORITY parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public int[] getPriority() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.DYNAMICPRIORITY parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public int[] getDynamicPriority() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.NICE parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public int[] getNice() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.STATUS parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public int[] getStatus() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.MAXJOBS parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public long[] getMaxJobs() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.MAXJOBSRUNNING parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public long[] getMaxJobsRunning() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.MAXJOBSPENDING parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public long[] getMaxJobsPending() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.MAXWALLTIME parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public long[] getMaxWallTime() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.MAXCPUTIME parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public long[] getMaxCpuTime() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.JOBSCOUNT parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public long[] getJobsCount() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.RUNNINGJOBSCOUNT parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public long[] getRunningJobsCount() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of {@link QueueParameterName}.WAITINGJOBSCOUNT parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */	
	public long[] getWaitingJobsCount() throws NoSuchParamException;

	/**
	 * 
	 * @return value of {@link QueueParameterName}.JOBWAITINGTIME parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public long[] getJobWaitingTime() throws NoSuchParamException;
	
	/**
	 * @param paramName name of the requested parameter.
	 * @return parameter object. This is the object representation of the "queueParameter" element in schema.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public QueueParameterType getParam(QueueParameterName paramName) throws NoSuchParamException;
	
}
