package org.qcg.broker.schemas.hostparams.wrapper.impl;

import org.qcg.broker.schemas.exception.NoSuchParamException;
import org.qcg.broker.schemas.hostparams.ComputingResource;
import org.qcg.broker.schemas.hostparams.ComputingResourceNetParameterType;
import org.qcg.broker.schemas.hostparams.ComputingResourceParameterType;
import org.qcg.broker.schemas.hostparams.ComputingResourceType;
import org.qcg.broker.schemas.hostparams.ComputingResourceTypeItem;
import org.qcg.broker.schemas.hostparams.OtherParameterType;
import org.qcg.broker.schemas.hostparams.types.ComputingParameterName;
import org.qcg.broker.schemas.hostparams.types.NetworkParameterName;
import org.qcg.broker.schemas.hostparams.wrapper.ComputingResourceParameters;

/**
 * 
 * @author Marcin Krystek
 *
 */
public class ComputingResourceParamExplorerImpl implements ComputingResourceParameters {

	protected ComputingResource computingResource;
	
	public ComputingResourceParameterType getHostParam(ComputingParameterName paramName) throws NoSuchParamException{
		if(computingResource == null)
			throw new NoSuchParamException("Computing resource is unavailable. " +
											"Please set computing resource before exploring its values.");
		
		boolean found = false;
		ComputingResourceType machineParameters = null;
		ComputingResourceTypeItem crItem = null;
		ComputingResourceParameterType hostParam = null;
		
		machineParameters = computingResource.getMachineParameters();
		
		for(int i = 0; i < machineParameters.getComputingResourceTypeItemCount() && !found; i++){
			crItem = machineParameters.getComputingResourceTypeItem(i);
			
			if((hostParam = crItem.getHostParameter()) != null) {
				
				if(hostParam.getName().equals(paramName)){
					found = true;
				}
			}
		}
		
		if(!found)
			throw new NoSuchParamException(paramName, "Parameter is not defined.");
		
		return hostParam;
		
	}
	
	public ComputingResourceNetParameterType getNetParam(NetworkParameterName paramName) throws NoSuchParamException {
		if(computingResource == null)
			throw new NoSuchParamException("Computing resource is unavailable. " +
											"Please set computing resource before exploring its values.");
		
		boolean found = false;
		ComputingResourceType machineParameters = null;
		ComputingResourceTypeItem crItem = null;
		ComputingResourceNetParameterType netParam = null;
		
		machineParameters = computingResource.getMachineParameters();
		
		for(int i = 0; i < machineParameters.getComputingResourceTypeItemCount() && !found; i++){
			crItem = machineParameters.getComputingResourceTypeItem(i);
			
			if((netParam = crItem.getNetworkParameter()) != null){
				
				if(netParam.getName().equals(paramName)){
					found = true;
				}
			}
		}
		
		if(!found)
			throw new NoSuchParamException(paramName, "Parameter is not defined.");
		
		return netParam;
	}
	
	public OtherParameterType getOtherParam(String paramName) throws NoSuchParamException{
		if(computingResource == null)
			throw new NoSuchParamException("Computing resource is unavailable. " +
											"Please set computing resource before exploring its values.");
		
		boolean found = false;
		ComputingResourceType machineParameters = null;
		ComputingResourceTypeItem crItem = null;
		OtherParameterType otherParam = null;
		
		machineParameters = computingResource.getMachineParameters();
		
		for(int i = 0; i < machineParameters.getComputingResourceTypeItemCount() && !found; i++){
			crItem = machineParameters.getComputingResourceTypeItem(i);
			
			if((otherParam = crItem.getOtherParameter()) != null) {
				
				if(otherParam.getName().equals(paramName)){
					found = true;
				}
			}
				
		}
		
		if(!found)
			throw new NoSuchParamException(paramName, "Parameter " + paramName + " is not defined.");
		
		return otherParam;
	}
	
	protected String[] getParamValue(ComputingParameterName paramName) throws NoSuchParamException{
		String value[] = null;
		ComputingResourceParameterType hostParam = null;
		
		hostParam = getHostParam(paramName);
		value = hostParam.getParamValue();
		
		return value;
	}
	
	protected String[] getParamValue(NetworkParameterName paramName) throws NoSuchParamException{
		String value[] = null;
		ComputingResourceNetParameterType netParam = null;
		
		netParam = getNetParam(paramName);
		value = netParam.getParamValue();
		
		return value;
	}
	
	
	public String[] getApplicationName() throws NoSuchParamException {
		return getParamValue(ComputingParameterName.APPLICATION);
	}

	public String[] getCpuarch() throws NoSuchParamException {
		return getParamValue(ComputingParameterName.CPUARCH);
	}

	public double[] getCpucount() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(ComputingParameterName.CPUCOUNT);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}

	public double[] getCpuLoad() throws NoSuchParamException {
		String sVal[] = null;
		double dVal[] = null;
		
		sVal = getParamValue(ComputingParameterName.CPULOAD);
		dVal = new double[sVal.length];
		
		for(int i = 0; i < dVal.length; i++){
			dVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return dVal;
	}

	public double[] getCpuspeed() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(ComputingParameterName.CPUSPEED);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}

	public double[] getDiskspace() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(ComputingParameterName.DISKSPACE);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}

	public double[] getFreecpus() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(ComputingParameterName.FREECPUS);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}

	public double[] getFreediskspace() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(ComputingParameterName.FREEDISKSPACE);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}

	public double[] getFreememory() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(ComputingParameterName.FREEMEMORY);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}

	public String[] getHostname() throws NoSuchParamException {
		return getParamValue(ComputingParameterName.HOSTNAME);
	}

	public String[] getLocalResourceManager() throws NoSuchParamException {
		return getParamValue(ComputingParameterName.LOCALRESOURCEMANAGER);
	}

	public double[] getMemory() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(ComputingParameterName.MEMORY);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}

	public String[] getOsname() throws NoSuchParamException {
		return getParamValue(ComputingParameterName.OSNAME);
	}

	public String[] getOsrelease() throws NoSuchParamException {
		return getParamValue(ComputingParameterName.OSRELEASE);
	}

	public String[] getOstype() throws NoSuchParamException {
		return getParamValue(ComputingParameterName.OSTYPE);
	}

	public String[] getOsversion() throws NoSuchParamException {
		return getParamValue(ComputingParameterName.OSVERSION);
	}

	public String[] getRemoteSubmissionInterface() throws NoSuchParamException {
		return getParamValue(ComputingParameterName.REMOTESUBMISSIONINTERFACE);
	}
	public double[] getBandwidth() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(NetworkParameterName.BANDWIDTH);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}
	
	public double[] getLatency() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(NetworkParameterName.LATENCY);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}
		
	public double[] getCapacity() throws NoSuchParamException {
		String sVal[] = null;
		double lVal[] = null;
		
		sVal = getParamValue(NetworkParameterName.CAPACITY);
		lVal = new double[sVal.length];
		
		for(int i = 0; i < lVal.length; i++){
			lVal[i] = Double.parseDouble(sVal[i]);
		}
		
		return lVal;
	}	

	public void explore(ComputingResource computingResource) {
		this.computingResource = computingResource;
	}

}
