package org.qcg.broker.schemas.hostparams.wrapper.impl;

import java.util.ArrayList;

import org.qcg.broker.schemas.exception.UnknownParameter;
import org.qcg.broker.schemas.hostparams.ComputingResource;
import org.qcg.broker.schemas.hostparams.FrontendInfo;
import org.qcg.broker.schemas.hostparams.HostParameters;
import org.qcg.broker.schemas.hostparams.NodeInfo;
import org.qcg.broker.schemas.hostparams.QueueResource;
import org.qcg.broker.schemas.hostparams.Resources;
import org.qcg.broker.schemas.hostparams.ResourcesItem;
import org.qcg.broker.schemas.hostparams.wrapper.HostParametersWrapper;




/**
 * 
 * @author Marcin Krystek
 *
 */

public class HostParametersWrapperImpl implements HostParametersWrapper{

	protected HostParameters hostParams;
	
	public void wrap(HostParameters hostParams) {
		this.hostParams = hostParams;
	}
	
	public ComputingResource[] getComputingResources() throws UnknownParameter {
		if(hostParams == null)
			throw new UnknownParameter("Host parameters are not defined.");
		
		ArrayList<ComputingResource>tab = null;
		Resources resources = null;
		ResourcesItem item = null;
		ComputingResource compRes = null;
		
		resources = hostParams.getResources();
		if(resources == null) return null;
		
		tab = new ArrayList<ComputingResource>();
		
		for(int i = 0; i < resources.getResourcesItemCount(); i++){
			item = resources.getResourcesItem(i);
			compRes = item.getComputingResource();
			
			if(compRes == null) continue;
			
			tab.add(compRes);
		}
		
		if(tab.size() == 0)
			return null;
		else
			return tab.toArray(new ComputingResource[0]);
	}

	public QueueResource[] getQueueResources() throws UnknownParameter  {
		if(hostParams == null)
			throw new UnknownParameter("Host parameters are not defined.");
		
		ArrayList<QueueResource> tab = null;
		Resources resources = null;
		ResourcesItem item = null;
		QueueResource queueResource = null;
		
		resources = hostParams.getResources();
		if(resources == null) return null;
		
		tab = new ArrayList<QueueResource>();
		
		for(int i = 0; i < resources.getResourcesItemCount(); i++){
			item = resources.getResourcesItem(i);
			queueResource = item.getQueueResource();
			
			if(queueResource == null) continue;
			
			tab.add(queueResource);
		}
		
		if(tab.size() == 0)
			return null;
		else
			return tab.toArray(new QueueResource[0]);
	}

	public ComputingResource[] getFrontends() throws UnknownParameter {
		if(hostParams == null)
			throw new UnknownParameter("Host parameters are not defined.");
		
		ArrayList <ComputingResource>frontends = null;
		ComputingResource compResources[] = null;
		ComputingResource compResource = null;
		
		compResources = getComputingResources();
		
		if(compResources == null)
			return null;
		
		frontends = new ArrayList<ComputingResource>();
		
		for(int i = 0; i < compResources.length; i++){
			compResource = compResources[i];
			if(compResource.getFrontendInfo() != null)
				frontends.add(compResource);
		}
		
		if(frontends.size() == 0)
			return null;
		else 
			return frontends.toArray(new ComputingResource[0]);
	}
	

	public ComputingResource getComputingResource(String id)
			throws UnknownParameter {
		
		ComputingResource compResources[] = null;
		
		if(id == null)
			return null;
		
		compResources = getComputingResources();
		if(compResources == null) 
			return null;
		
		for(int i = 0; i < compResources.length; i++){
			if(id.equals(compResources[i].getResourceId()))
				return compResources[i];
		}
		
		return null;
	}
	
	public ComputingResource[] getStandaloneComputingResources() throws UnknownParameter{
		ArrayList<ComputingResource> ret = new ArrayList<ComputingResource>();
		ComputingResource tab[] = null;
		ComputingResource resource = null;
		FrontendInfo frontendInf = null;
		NodeInfo nodeInf = null;
		boolean add = true;
		
		tab = getComputingResources();
		if(tab == null) return null;
		
		for(int i = 0; i < tab.length; i++){
			resource  = tab[i];
			frontendInf = resource.getFrontendInfo();
			nodeInf = resource.getNodeInfo();
			add = true;
			
			if(frontendInf != null){
				for(int j = 0; j < frontendInf.getInterfaceCount() && add; j++){
					if(frontendInf.getInterface(j).getQueueResourceIdRefCount() != 0)
						add = false;
				}
			}
			
			if(add && nodeInf != null){
				if(add && nodeInf.getFrontendResourceIdRefCount() != 0)
					add = false;
				if(add && nodeInf.getQueueResourceIdRefCount() != 0)
					add = false;
			}
			
			if(add){
				ret.add(resource);
			}
		}
		
		if(ret.size() == 0)
			return null;
		else
			return ret.toArray(new ComputingResource[0]);
	}

	public QueueResource getQueueResource(String id)
			throws UnknownParameter {
		
		QueueResource queueResources[] =  null;
		
		if(id == null)
			return null;
		
		queueResources = getQueueResources();
		if(queueResources == null)
			return null;
		
		for(int i = 0; i < queueResources.length; i++){
			if(id.equals(queueResources[i].getResourceId()))
				return queueResources[i];
		}
		
		return null;
	}


}
