package org.qcg.broker.schemas.hostparams.wrapper.impl;

import org.qcg.broker.schemas.exception.NoSuchParamException;
import org.qcg.broker.schemas.hostparams.QueueParameterType;
import org.qcg.broker.schemas.hostparams.QueueResource;
import org.qcg.broker.schemas.hostparams.QueueResourceType;
import org.qcg.broker.schemas.hostparams.QueueResourceTypeItem;
import org.qcg.broker.schemas.hostparams.types.QueueParameterName;
import org.qcg.broker.schemas.hostparams.wrapper.QueueResourceParamExplorer;

/**
 * 
 * @author Marcin Krystek
 *
 */
public class QueueResourceParamExplorerImpl implements QueueResourceParamExplorer {

	protected QueueResource queueResource;
	
	public QueueParameterType getParam(QueueParameterName paramName) throws NoSuchParamException{
		if(queueResource == null)
			throw new NoSuchParamException("Queue resource is unavailable. " +
										"Please set queue resource before exploring its values."); 
		
		boolean found = false;
		QueueResourceType resParam = null;
		QueueParameterType param = null;
		
		for(int i = 0; i < queueResource.getQueueParametersCount() && !found; i++){
			resParam = queueResource.getQueueParameters(i);
			if(resParam == null)
				continue;
			
			for(int j = 0; j < resParam.getQueueResourceTypeItemCount() && !found; j++){
				QueueResourceTypeItem item = resParam.getQueueResourceTypeItem(j);
				param = item.getQueueParameter();
				
				if(param.getName().equals(paramName)){
					found = true;
				}
			}
			
		}
		
		if(!found)
			throw new NoSuchParamException(paramName, "Parameter is not defined.");
		
		return param;
	}
	
	protected String[] getParamValue(QueueParameterName paramName) throws NoSuchParamException{
		String value[] = null;
		QueueParameterType param = null;
		
		param = getParam(paramName);
		value = param.getParamValue();
		
		return value;
	}
	
	public int[] getDynamicPriority() throws NoSuchParamException {
		String sValue[] = null;
		int iValue[] = null;
		
		sValue = getParamValue(QueueParameterName.DYNAMICPRIORITY);
		iValue = new int[sValue.length];
		
		for(int i = 0; i < iValue.length; i++){
			iValue[i] = Integer.parseInt(sValue[i]);
		}
		
		return iValue;
	}

	public long[] getJobWaitingTime() throws NoSuchParamException {
		String sValue[] = null;
		long lValue[] = null;
		
		sValue = getParamValue(QueueParameterName.JOBWAITINGTIME);
		lValue = new long[sValue.length];
		
		for(int i = 0; i < lValue.length; i++){
			lValue[i] = Long.parseLong(sValue[i]);
		}
		
		return lValue;
	}

	public long[] getJobsCount() throws NoSuchParamException {
		String sValue[] = null;
		long lValue[] = null;
		
		sValue = getParamValue(QueueParameterName.JOBSCOUNT);
		lValue = new long[sValue.length];
		
		for(int i = 0; i < lValue.length; i++){
			lValue[i] = Long.parseLong(sValue[i]);
		}
		
		return lValue;
	}

	public long[] getMaxCpuTime() throws NoSuchParamException {
		String sValue[] = null;
		long lValue[] = null;
		
		sValue = getParamValue(QueueParameterName.MAXCPUTIME);
		lValue = new long[sValue.length];
		
		for(int i = 0; i < lValue.length; i++){
			lValue[i] = Long.parseLong(sValue[i]);
		}
		
		return lValue;
	}

	public long[] getMaxJobs() throws NoSuchParamException {
		String sValue[] = null;
		long lValue[] = null;
		
		sValue = getParamValue(QueueParameterName.MAXJOBS);
		lValue = new long[sValue.length];
		
		for(int i = 0; i < lValue.length; i++){
			lValue[i] = Long.parseLong(sValue[i]);
		}
		
		return lValue;
	}

	public long[] getMaxJobsPending() throws NoSuchParamException {
		String sValue[] = null;
		long lValue[] = null;
		
		sValue = getParamValue(QueueParameterName.MAXJOBSPENDING);
		lValue = new long[sValue.length];
		
		for(int i = 0; i < lValue.length; i++){
			lValue[i] = Long.parseLong(sValue[i]);
		}
		
		return lValue;
	}

	public long[] getMaxJobsRunning() throws NoSuchParamException {
		String sValue[] = null;
		long lValue[] = null;
		
		sValue = getParamValue(QueueParameterName.MAXJOBSRUNNING);
		lValue = new long[sValue.length];
		
		for(int i = 0; i < lValue.length; i++){
			lValue[i] = Long.parseLong(sValue[i]);
		}
		
		return lValue;
	}

	public long[] getMaxWallTime() throws NoSuchParamException {
		String sValue[] = null;
		long lValue[] = null;
		
		sValue = getParamValue(QueueParameterName.MAXWALLTIME);
		lValue = new long[sValue.length];
		
		for(int i = 0; i < lValue.length; i++){
			lValue[i] = Long.parseLong(sValue[i]);
		}
		
		return lValue;
	}

	public int[] getNice() throws NoSuchParamException {
		String sValue[] = null;
		int iValue[] = null;
		
		sValue = getParamValue(QueueParameterName.NICE);
		iValue = new int[sValue.length];
		
		for(int i = 0; i < iValue.length; i++){
			iValue[i] = Integer.parseInt(sValue[i]);
		}
		
		return iValue;
	}

	public int[] getPriority() throws NoSuchParamException {
		String sValue[] = null;
		int iValue[] = null;
		
		sValue = getParamValue(QueueParameterName.PRIORITY);
		iValue = new int[sValue.length];
		
		for(int i = 0; i < iValue.length; i++){
			iValue[i] = Integer.parseInt(sValue[i]);
		}
		
		return iValue;
	}

	public String[] getQueueName() throws NoSuchParamException {
		return getParamValue(QueueParameterName.QUEUENAME);
	}

	public long[] getRunningJobsCount() throws NoSuchParamException {
		String sValue[] = null;
		long lValue[] = null;
		
		sValue = getParamValue(QueueParameterName.RUNNINGJOBSCOUNT);
		lValue = new long[sValue.length];
		
		for(int i = 0; i < lValue.length; i++){
			lValue[i] = Long.parseLong(sValue[i]);
		}
		
		return lValue;
	}

	public int[] getStatus() throws NoSuchParamException {
		String sValue[] = null;
		int iValue[] = null;
		
		sValue = getParamValue(QueueParameterName.STATUS);
		iValue = new int[sValue.length];
		
		for(int i = 0; i < iValue.length; i++){
			iValue[i] = Integer.parseInt(sValue[i]);
		}
		
		return iValue;
	}

	public long[] getWaitingJobsCount() throws NoSuchParamException {
		String sValue[] = null;
		long lValue[] = null;
		
		sValue = getParamValue(QueueParameterName.WAITINGJOBSCOUNT);
		lValue = new long[sValue.length];
		
		for(int i = 0; i < lValue.length; i++){
			lValue[i] = Long.parseLong(sValue[i]);
		}
		
		return lValue;
	}

	public void explore(QueueResource queueResource) {
		this.queueResource = queueResource;
	}

}
