package org.qcg.broker.schemas.hostparams.wrapper.impl;


import java.util.ArrayList;

import org.qcg.broker.schemas.exception.NoSuchParamException;
import org.qcg.broker.schemas.exception.UnknownParameter;
import org.qcg.broker.schemas.hostparams.ComputingResource;
import org.qcg.broker.schemas.hostparams.ComputingResourceNetParameterType;
import org.qcg.broker.schemas.hostparams.ComputingResourceParameterType;
import org.qcg.broker.schemas.hostparams.FrontendInfo;
import org.qcg.broker.schemas.hostparams.HostParameters;
import org.qcg.broker.schemas.hostparams.Interface;
import org.qcg.broker.schemas.hostparams.NodeInfo;
import org.qcg.broker.schemas.hostparams.OtherParameterType;
import org.qcg.broker.schemas.hostparams.QueueResource;
import org.qcg.broker.schemas.hostparams.Resources;
import org.qcg.broker.schemas.hostparams.ResourcesItem;
import org.qcg.broker.schemas.hostparams.StatValuesType;
import org.qcg.broker.schemas.hostparams.types.ComputingParameterName;
import org.qcg.broker.schemas.hostparams.types.NetworkParameterName;
import org.qcg.broker.schemas.hostparams.types.QueueParameterName;
import org.qcg.broker.schemas.hostparams.wrapper.QueueResourceParamExplorer;
import org.qcg.broker.schemas.hostparams.wrapper.QueuingSystem;
import org.qcg.broker.schemas.hostparams.wrapper.exception.NotHomogeneous;
import org.qcg.broker.schemas.utils.ComputingResourceParamExplorer;



/**
 * 
 * @author Marcin Krystek
 *
 */
public class QueuingSystemImpl implements QueuingSystem{

	protected HostParameters hostParams;
	protected ComputingResource frontend;
	protected boolean isHomogeneous;
	
	public QueuingSystemImpl() {
		this.hostParams = null;
		this.frontend = null;
		this.isHomogeneous = false;
	}
	
	public QueuingSystemImpl(HostParameters hostParams, ComputingResource frontend){
		this.hostParams = hostParams;
		this.frontend = frontend;
		
		this.isHomogeneous = checkHomogenity();
		
	}
	
	public void create(HostParameters hostParams, ComputingResource frontend){
		this.hostParams = hostParams;
		this.frontend = frontend;
		
		this.isHomogeneous = checkHomogenity();
	}
	
	public ComputingResource getFrontend(){
		return frontend;
	}
	
	public double getAvgFreeMemory() throws NoSuchParamException, UnknownParameter {
		return computeAvgValue(ComputingParameterName.FREEMEMORY);
	}

	public double getAvgCpuLoad() throws NoSuchParamException, UnknownParameter {
		return computeAvgValue(ComputingParameterName.CPULOAD);
	}

	public double getBandwidth() throws NoSuchParamException, UnknownParameter, NotHomogeneous {
		return getLongParamValue(NetworkParameterName.BANDWIDTH);
	}

	public double getCapacity() throws NoSuchParamException, UnknownParameter, NotHomogeneous {
		return getLongParamValue(NetworkParameterName.CAPACITY);
	}

	public ComputingResource[] getComputingNodes() throws UnknownParameter {
		if(hostParams == null)
			throw new UnknownParameter();
		
		if(frontend == null)
			throw new UnknownParameter("Unable to identify frontend for queueing system.");
		
		String frontendId = null;
		String frontendQueueId = null;
		String frontendRefId = null;
		String queueIdRef = null;
		ComputingResource compRes = null;
		ArrayList <ComputingResource>tab = null;
		Resources resources = null;
		ResourcesItem resItem = null;
		NodeInfo nodeInfo = null;
		FrontendInfo frontendInfo = null;
		Interface intrf = null;
		
		boolean foundCompRes = false;
		boolean foundQueue = false;
		
		resources = hostParams.getResources();
		if(resources == null)
			return null;
		
		tab = new ArrayList<ComputingResource>();
		frontendId = frontend.getResourceId();
		
		for(int i = 0; i < resources.getResourcesItemCount(); i++){
			resItem = resources.getResourcesItem(i);
			compRes = resItem.getComputingResource();
			if(compRes == null) continue;
			
			nodeInfo = compRes.getNodeInfo();
			if(nodeInfo == null) continue;
			
			foundCompRes = false;
			foundQueue = false;

			// check if computingResource refers to this frontend
			for(int j = 0; j < nodeInfo.getFrontendResourceIdRefCount() && !foundCompRes; j++){
				frontendRefId = nodeInfo.getFrontendResourceIdRef(j);
				if(frontendId.equals(frontendRefId)){
					foundCompRes = true;
				}
			}
			
			frontendInfo = frontend.getFrontendInfo();
			
			// check if computing resource refers to the queue in frontend queue list
			for(int j = 0; j < nodeInfo.getQueueResourceIdRefCount() && foundCompRes && !foundQueue; j++){
				queueIdRef = nodeInfo.getQueueResourceIdRef(j);
				
				for(int interfaceIdx = 0; interfaceIdx < frontendInfo.getInterfaceCount() && !foundQueue; interfaceIdx++){
					intrf = frontendInfo.getInterface(interfaceIdx);
					for(int queueIdx = 0; queueIdx < intrf.getQueueResourceIdRefCount() && !foundQueue; queueIdx++){
						frontendQueueId = intrf.getQueueResourceIdRef(queueIdx);
						if(frontendQueueId.equals(queueIdRef))
							foundQueue = true;
					}
				}
			}

			if(foundCompRes && foundQueue){
				tab.add(compRes);
			}
			
		}
		
		if(tab.size() == 0)
			return null;
		else
			return tab.toArray(new ComputingResource[0]);
		
	}

	public ComputingResource[] getComputingNodes(QueueResource queueResource)
			throws UnknownParameter {
		if(queueResource == null)
			return null;
		
		String queueId = null;
		String refId = null;
		ComputingResource compResources[] = null;
		ComputingResource compRes = null;
		NodeInfo nodeInfo = null;
		ArrayList<ComputingResource> tab = null;
		
		compResources = getComputingNodes();
		
		if(compResources == null)
			return null;

		tab = new ArrayList<ComputingResource>();
		queueId = queueResource.getResourceId();
		
		for(int i = 0; i < compResources.length; i++){
			compRes = compResources[i];
			nodeInfo = compRes.getNodeInfo();
			
			if(nodeInfo == null) continue;
			
			for(int j = 0; j < nodeInfo.getQueueResourceIdRefCount(); j++){
				refId = nodeInfo.getQueueResourceIdRef(j);
				if(queueId.equals(refId)){
					tab.add(compRes);
					break;
				}
			}
		}
		
		if(tab.size() == 0)
			return null;
		else
			return tab.toArray(new ComputingResource[0]);
	}
	
	public String getCpuArch() throws NoSuchParamException, UnknownParameter, NotHomogeneous{
		return getStringParamValue(ComputingParameterName.CPUARCH);
	}

	public double getFreeCpus() throws NoSuchParamException, UnknownParameter {
		return computeSumValue(ComputingParameterName.FREECPUS);
	}

	public double getMemory() throws NoSuchParamException, UnknownParameter {
		return computeSumValue(ComputingParameterName.MEMORY);
	}
	
	public double getFreeMemory() throws NoSuchParamException, UnknownParameter {
		return computeSumValue(ComputingParameterName.FREEMEMORY);
	}

	public String getFrontendId() {
		return frontend.getResourceId();
	}

	public double getLatency() throws NoSuchParamException, UnknownParameter, NotHomogeneous {
		return getLongParamValue(NetworkParameterName.LATENCY);
	}

	public double getCpuSpeed() throws NoSuchParamException, UnknownParameter, NotHomogeneous {
		return getLongParamValue(ComputingParameterName.CPUSPEED);
	}

	public String getOsName() throws NoSuchParamException, UnknownParameter, NotHomogeneous {
		return getStringParamValue(ComputingParameterName.OSNAME);
	}

	public String getOsRelease() throws NoSuchParamException, UnknownParameter, NotHomogeneous {
		return getStringParamValue(ComputingParameterName.OSRELEASE);
	}

	public String getOsType() throws NoSuchParamException, UnknownParameter, NotHomogeneous {
		return getStringParamValue(ComputingParameterName.OSTYPE);
	}

	public String getOsVersion() throws NoSuchParamException, UnknownParameter, NotHomogeneous {
		return getStringParamValue(ComputingParameterName.OSVERSION);
	}

	public QueueResource[] getQueues() throws UnknownParameter {
		if(hostParams == null)
			throw new UnknownParameter();
		
		if(frontend == null)
			throw new UnknownParameter("Unable to identify frontend for queueing system.");
		
		Resources resources = null;
		ResourcesItem resItem = null;
		QueueResource queueResource = null;
		ArrayList<QueueResource>tab = null;
		FrontendInfo frontendInfo = null;
		boolean foundQueue = false;
		Interface intrf = null;
		String frontendQueueId = null;
		
		resources = hostParams.getResources();
		if(resources == null) return null;
		
		frontendInfo = frontend.getFrontendInfo();
		tab = new ArrayList<QueueResource>();
		for(int i = 0; i < resources.getResourcesItemCount(); i++){
			resItem = resources.getResourcesItem(i);
			queueResource = resItem.getQueueResource();
			if(queueResource == null) continue;
			
			// check if queue is on the frontend queue list
				foundQueue = false;
				for(int interfaceIdx = 0; interfaceIdx < frontendInfo.getInterfaceCount() && !foundQueue; interfaceIdx++){
					intrf = frontendInfo.getInterface(interfaceIdx);
					for(int queueIdx = 0; queueIdx < intrf.getQueueResourceIdRefCount() && !foundQueue; queueIdx++){
						frontendQueueId = intrf.getQueueResourceIdRef(queueIdx);
						if(frontendQueueId.equals(queueResource.getResourceId()))
							foundQueue = true;
					}
				}
				
				if(foundQueue)
					tab.add(queueResource);
			
		}
		
		if(tab.size() == 0)
			return null;
		else
			return tab.toArray(new QueueResource[0]);
	}

	public double getRunningJobsCount() throws NoSuchParamException, UnknownParameter {
		return computeSumValue(QueueParameterName.RUNNINGJOBSCOUNT);
	}

	public double getCpuCnt() throws NoSuchParamException, UnknownParameter {
		return computeSumValue(ComputingParameterName.CPUCOUNT);
	}

	public double getJobsCount() throws NoSuchParamException, UnknownParameter {
		return computeSumValue(QueueParameterName.JOBSCOUNT);
	}
	
	public double getWaitingJobsCount() throws NoSuchParamException, UnknownParameter{
		return computeSumValue(QueueParameterName.WAITINGJOBSCOUNT);
	}

	/**
	 * Queuing system is homogeneous if all computing resources have equal values for 
	 * following parameters: {@link ComputingParameterName}.OSNAME, 
	 * {@link ComputingParameterName}.OSTYPE, {@link ComputingParameterName}.CPUARCH, 
	 * {@link ComputingParameterName}.OSVERSION, {@link ComputingParameterName}.OSRELEASE, 
	 * {@link ComputingParameterName}.MEMORY, {@link ComputingParameterName}.CPUCOUNT, 
	 * {@link ComputingParameterName}.CPUSPEED, {@link NetworkParameterName}.BANDWIDTH.
	 * If some this parameters is undefined, then it must be undefined for all computing resources.
	 */
	public boolean isHomogeneous() {
		return isHomogeneous;
	}

	
	
	protected double computeAvgValue(ComputingParameterName paramSelector) throws UnknownParameter, NoSuchParamException{
		ComputingResource compResources[] = null;
		ComputingResourceParamExplorer<ComputingResource, ComputingResourceNetParameterType, 
		ComputingResourceParameterType, OtherParameterType, 
		ComputingParameterName, NetworkParameterName> explorer = null;
		ComputingResourceParameterType hostParams = null;
		StatValuesType stats = null;
		double retValue = 0;
		double localAvg = 0;
		
		compResources = getComputingNodes();
		if(compResources == null || compResources.length == 0)
			throw new UnknownParameter();
	
		explorer = new ComputingResourceParamExplorerImpl();
		
		for(int i = 0; i < compResources.length; i++){
			explorer.explore(compResources[i]);
			explorer.getOsname();
			hostParams = explorer.getHostParam(paramSelector);
			localAvg = 0;
		
			for(int j = 0; j < hostParams.getStatValuesCount(); j++){
				stats = hostParams.getStatValues(j);
				if(stats.hasAvg()){
					localAvg += stats.getAvg();
				}
			}
			
			if(hostParams.getParamValueCount() != 0)
				localAvg = localAvg / hostParams.getStatValuesCount();
			
			retValue += localAvg;
		}
		
		retValue = retValue / compResources.length;
		return retValue;
	}
	
	
	
	protected double computeSumValue(ComputingParameterName paramSelector) throws NoSuchParamException, UnknownParameter{
		ComputingResource compResources[] = null;
		ComputingResourceParamExplorer explorer = null;
		double retValue = 0;
		double paramValues[] = null;
		
		compResources = getComputingNodes();
		if(compResources == null || compResources.length == 0) 
			throw new UnknownParameter();
		
		explorer = new ComputingResourceParamExplorerImpl();
		for(int i = 0; i < compResources.length; i++){
			explorer.explore(compResources[i]);

			switch(paramSelector) {
				
				case MEMORY:
					paramValues = explorer.getMemory();
					break;
				case FREEMEMORY:
					paramValues = explorer.getFreememory();
					break;
				case CPUCOUNT:
					paramValues = explorer.getCpucount();
					break;
				case FREECPUS:
					paramValues = explorer.getFreecpus();
					break;
				case CPUSPEED:
					paramValues = explorer.getCpuspeed();
					break;
				case DISKSPACE:
					paramValues = explorer.getDiskspace();
					break;
				case FREEDISKSPACE:
					paramValues = explorer.getFreediskspace();
					break;
				default:
					throw new NoSuchParamException("Parameter not supported.");
				
			}
			
			retValue += paramValues[0];
			
		}
		
		return retValue;
	}
	
	protected long computeSumValue(QueueParameterName paramSelector) throws NoSuchParamException, UnknownParameter{
		QueueResource queueResources[] = null;
		QueueResourceParamExplorer explorer = null;
		long retValue = 0;
		long paramValues[] = null;
		
		queueResources = getQueues();
		if(queueResources == null || queueResources.length == 0) 
			throw new UnknownParameter();
		
		explorer = new QueueResourceParamExplorerImpl();
		
		for(int i = 0; i < queueResources.length; i++){
			explorer.explore(queueResources[i]);

			switch(paramSelector) {
				
				case JOBSCOUNT:
					paramValues = explorer.getJobsCount();
					break;
				case RUNNINGJOBSCOUNT:
					paramValues = explorer.getRunningJobsCount();
					break;
				case WAITINGJOBSCOUNT:
					paramValues = explorer.getWaitingJobsCount();
					break;
				default:
					throw new NoSuchParamException("Parameter not supported.");
				
			}
			
			retValue += paramValues[0];
			
		}
		
		return retValue;
	}
	
	
	protected double getLongParamValue(ComputingParameterName paramSelector) 
				throws NotHomogeneous, NoSuchParamException, UnknownParameter{
		ComputingResource compResources[] = null;
		ComputingResourceParamExplorer explorer = null;
		double paramValues[] = null;
		
		if(!isHomogeneous()){
			throw new NotHomogeneous();
		}
		
		compResources = getComputingNodes();
		if(compResources == null || compResources.length == 0)
			throw new UnknownParameter();

		explorer = new ComputingResourceParamExplorerImpl();
		explorer.explore(compResources[0]);

		switch(paramSelector){
			case CPUSPEED:
				paramValues = explorer.getCpuspeed();
				break;
			default: throw new NoSuchParamException("Parameter not supported.");
		}
		
		return paramValues[0];
	}
	
	protected double getLongParamValue(NetworkParameterName paramSelector) 
			throws NotHomogeneous, NoSuchParamException, UnknownParameter{
		ComputingResource compResources[] = null;
		ComputingResourceParamExplorer explorer = null;
		double paramValues[] = null;
		
		if(!isHomogeneous()){
			throw new NotHomogeneous();
		}
		
		compResources = getComputingNodes();
		if(compResources == null || compResources.length == 0)
			throw new UnknownParameter();
		
		explorer = new ComputingResourceParamExplorerImpl();
		explorer.explore(compResources[0]);
		
		switch(paramSelector){
			case BANDWIDTH:
				paramValues = explorer.getBandwidth();
				break;
			case CAPACITY:
				paramValues = explorer.getCapacity();
				break;
			case LATENCY:
				paramValues = explorer.getLatency();
				break;
			default: throw new NoSuchParamException("Parameter not supported.");
		}
		
		return paramValues[0];
	}
	
	protected String getStringParamValue(ComputingParameterName paramSelector) 
				throws NotHomogeneous, NoSuchParamException, UnknownParameter{
		ComputingResource compResources[] = null;
		ComputingResourceParamExplorer explorer = null;
		String paramValues[] = null;
		
		if(!isHomogeneous()){
			throw new NotHomogeneous();
		}
		
		compResources = getComputingNodes();
		if(compResources == null || compResources.length == 0)
			throw new UnknownParameter();

		explorer = new ComputingResourceParamExplorerImpl();
		explorer.explore(compResources[0]);

		switch(paramSelector){
			case OSNAME:
				paramValues = explorer.getOsname();
				break;
			case OSRELEASE:
				paramValues = explorer.getOsrelease();
				break;
			case OSVERSION:
				paramValues = explorer.getOsversion();
				break;
			case OSTYPE:
				paramValues = explorer.getOstype();
				break;
			case CPUARCH:
				paramValues = explorer.getCpuarch();
				break;
			default: throw new NoSuchParamException("Parameter not supported.");
		}
		
		return paramValues[0];
	}
	
	
	protected boolean checkHomogenity(){
		if(hostParams == null || frontend == null)
			return false;
		
		ComputingResource compResources[] = null;
		ComputingResource referenceCompRes = null;
		
		try {
			compResources = getComputingNodes();
		} catch (UnknownParameter e) {
			return false;
		}
		
		// it there are no computing resources or there is only one, the system is homogeneous
		if(compResources == null || compResources.length < 2)
			return true;
		
		referenceCompRes = compResources[0];
		
		for(int i = 1; i < compResources.length; i++){
			if(!compare(referenceCompRes, compResources[i])){
				return false;
			}
		}
		
		return true;
	}
	
	/**
	 * 
	 * @param cr1
	 * @param cr2
	 * @return true if cr1 equals cr2, false otherwise
	 */
	protected boolean compare(ComputingResource cr1, ComputingResource cr2){
		ComputingResourceParamExplorer<ComputingResource, ComputingResourceNetParameterType, 
												ComputingResourceParameterType, OtherParameterType, 
												ComputingParameterName, NetworkParameterName> refExplorer = null;
		ComputingResourceParamExplorer<ComputingResource, ComputingResourceNetParameterType, 
												ComputingResourceParameterType, OtherParameterType, 
												ComputingParameterName, NetworkParameterName> cr2Explorer = null;
		String refValues[] = null;
		String cr2Values[] = null; 
		double refLongValues[] = null;
		double cr2LongValues[] = null; 
		boolean equal = true;
		
		refExplorer = new ComputingResourceParamExplorerImpl();
		refExplorer.explore(cr1);
		
		cr2Explorer = new ComputingResourceParamExplorerImpl();
		cr2Explorer.explore(cr2);

		// check osname
		try {
			refValues = null;
			cr2Values = null;
			refValues = refExplorer.getOsname();
			cr2Values = cr2Explorer.getOsname();
		} catch (NoSuchParamException e) {
		}
		equal = equal && compare(refValues, cr2Values);
		if(!equal) return false;
		
		// check ostype
		try {
			refValues = null;
			cr2Values = null;
			refValues = refExplorer.getOstype();
			cr2Values = cr2Explorer.getOstype();
		} catch (NoSuchParamException e) {
		}
		equal = equal && compare(refValues, cr2Values);
		if(!equal) return false;
		
		// check cpuarch
		try {
			refValues = null;
			cr2Values = null;
			refValues = refExplorer.getCpuarch();
			cr2Values = cr2Explorer.getCpuarch();
		} catch (NoSuchParamException e) {
		}
		equal = equal && compare(refValues, cr2Values);
		if(!equal) return false;
		
		// check osversion
		try {
			refValues = null;
			cr2Values = null;
			refValues = refExplorer.getOsversion();
			cr2Values = cr2Explorer.getOsversion();
		} catch (NoSuchParamException e) {
		}
		equal = equal && compare(refValues, cr2Values);
		if(!equal) return false;
		
		// check osrelease
		try {
			refValues = null;
			cr2Values = null;
			refValues = refExplorer.getOsrelease();
			cr2Values = cr2Explorer.getOsrelease();
		} catch (NoSuchParamException e) {
		}
		equal = equal && compare(refValues, cr2Values);
		if(!equal) return false;
		
		// check memory
		try {
			refLongValues = null;
			cr2LongValues = null;
			refLongValues = refExplorer.getMemory();
			cr2LongValues = cr2Explorer.getMemory();
		} catch (NoSuchParamException e) {
		}
		equal = equal && compare(refLongValues, cr2LongValues);
		if(!equal) return false;
		
		// check cpucount
		try {
			refLongValues = null;
			cr2LongValues = null;
			refLongValues = refExplorer.getCpucount();
			cr2LongValues = cr2Explorer.getCpucount();
		} catch (NoSuchParamException e) {
		}
		equal = equal && compare(refLongValues, cr2LongValues);
		if(!equal) return false;
		
		// check cpuspeed
		try {
			refLongValues = null;
			cr2LongValues = null;
			refLongValues = refExplorer.getCpuspeed();
			cr2LongValues = cr2Explorer.getCpuspeed();
		} catch (NoSuchParamException e) {
		}
		equal = equal && compare(refLongValues, cr2LongValues);
		if(!equal) return false;
		
		// check bandwidth
		try {
			ComputingResourceNetParameterType refNetParam = 
							refExplorer.getNetParam(NetworkParameterName.BANDWIDTH);
			ComputingResourceNetParameterType cr2NetParam = 
							cr2Explorer.getNetParam(NetworkParameterName.BANDWIDTH);
			
			refValues = null;
			cr2Values = null;
			refValues = refNetParam.getParamValue();
			cr2Values = cr2NetParam.getParamValue();
		} catch (NoSuchParamException e) {
		}
		equal = equal && compare(refValues, cr2Values);
		if(!equal) return false;
		
		return equal;
	}

	protected boolean compare(String tab1[], String tab2[]){
		if(tab1 == null && tab2 == null)
			return true;
		
		if((tab1 == null && tab2 != null) || (tab1 != null && tab2 == null))
			return false;
		
		if(tab1.length != tab2.length)
			return false;
		
		for(int i = 0; i < tab1.length; i++){
			if(!tab1[i].equals(tab2[i]))
				return false;
		}
		
		return true;
		
	}
	
	protected boolean compare(double tab1[], double tab2[]){
		if(tab1 == null && tab2 == null)
			return true;
		
		if((tab1 == null && tab2 != null) || (tab1 != null && tab2 == null))
			return false;
		
		if(tab1.length != tab2.length)
			return false;
		
		for(int i = 0; i < tab1.length; i++){
			if(tab1[i] != (tab2[i]))
				return false;
		}
		
		return true;
		
	}
}
