package org.qcg.broker.schemas.jobdesc.wrapper.impl;

import java.util.ArrayList;

import org.qcg.broker.schemas.exception.NoSuchParamException;
import org.qcg.broker.schemas.jobdesc.ComputingResourceBaseTypeChoice;
import org.qcg.broker.schemas.jobdesc.ComputingResourceBaseTypeChoiceItem;
import org.qcg.broker.schemas.jobdesc.ComputingResourceNetParameterType;
import org.qcg.broker.schemas.jobdesc.ComputingResourceParameterType;
import org.qcg.broker.schemas.jobdesc.ComputingResourceTemplate;
import org.qcg.broker.schemas.jobdesc.ComputingResourceTemplateItem;
import org.qcg.broker.schemas.jobdesc.ComputingResourceType;
import org.qcg.broker.schemas.jobdesc.OtherParameterType;
import org.qcg.broker.schemas.jobdesc.ParameterBoundaryType;
import org.qcg.broker.schemas.jobdesc.ParameterTypeChoice;
import org.qcg.broker.schemas.jobdesc.ParameterTypeChoiceItem;
import org.qcg.broker.schemas.jobdesc.Range;
import org.qcg.broker.schemas.jobdesc.ResourceTemplatesType;
import org.qcg.broker.schemas.jobdesc.ResourceTemplatesTypeItem;
import org.qcg.broker.schemas.jobdesc.StringValue;
import org.qcg.broker.schemas.jobdesc.Value;
import org.qcg.broker.schemas.jobdesc.types.ComputingParameterName;
import org.qcg.broker.schemas.jobdesc.types.NetworkParameterName;
import org.qcg.broker.schemas.jobdesc.wrapper.ComputingResourceJobDescParamExplorer;



/**
 * 
 * @author Marcin Krystek
 *
 */
public class ComputingResourceParamExplorerImpl implements ComputingResourceJobDescParamExplorer {

	protected ComputingResourceType computingResource;
	protected ResourceTemplatesType resourceTemplates;
	
	public ComputingResourceParameterType getHostParam(ComputingParameterName paramName) throws NoSuchParamException{
		if(computingResource == null)
			throw new NoSuchParamException("Computing resource is unavailable. " +
											"Please set computing resource before exploring its values.");
		
		boolean found = false;
		ComputingResourceBaseTypeChoiceItem crItem = null;
		ComputingResourceParameterType hostParam = null;
		ComputingResourceBaseTypeChoice choice = null;
		String refId = null;
		
		if((refId = computingResource.getTemplateIdReference()) != null){
			if(this.resourceTemplates == null)
				throw new NoSuchParamException(paramName, "Computing resource template is not defined.");
			
			ResourceTemplatesTypeItem item = null;
			ComputingResourceTemplate crt = null;
			
			// loop iterates over all templates
			for(int i = 0; i < this.resourceTemplates.getResourceTemplatesTypeItemCount() && !found; i++){
				item = this.resourceTemplates.getResourceTemplatesTypeItem(i);
				crt = item.getComputingResourceTemplate();
				
				// looking for correct template
				if(refId.equals(crt.getTemplateId())){
				
					ComputingResourceTemplateItem crtItem = null;
					
					// looking for correct parameter in template
					for(int j = 0; j < crt.getComputingResourceTemplateItemCount() && !found; j++){
						crtItem = crt.getComputingResourceTemplateItem(j);
						
						if((hostParam = crtItem.getHostParameter()) != null){
							
							if(hostParam.getName().equals(paramName)){
								found = true;
							}
						}
					}
				}
			}
			
		} else if((choice = computingResource.getComputingResourceBaseTypeChoice()) != null) {
			
			for(int i = 0; i < choice.getComputingResourceBaseTypeChoiceItemCount() && !found; i++){
				crItem = choice.getComputingResourceBaseTypeChoiceItem(i);
				if((hostParam = crItem.getHostParameter()) != null) {
					
					if(hostParam.getName().equals(paramName)){
						found = true;
					}
				}
			}
			
			
		}

		if(!found)
			throw new NoSuchParamException(paramName, "Parameter is not defined.");
		
		return hostParam;
	}
	
	public ComputingResourceNetParameterType getNetParam(NetworkParameterName paramName) throws NoSuchParamException {
		if(computingResource == null)
			throw new NoSuchParamException("Computing resource is unavailable. " +
											"Please set computing resource before exploring its values.");
		
		boolean found = false;
		ComputingResourceBaseTypeChoiceItem crItem = null;
		ComputingResourceNetParameterType netParam = null;
		String refId = null;
		ComputingResourceBaseTypeChoice choice = null;
		
		if((refId = computingResource.getTemplateIdReference()) != null){
			if(this.resourceTemplates == null)
				throw new NoSuchParamException(paramName, "Computing resource template is not defined.");
			
			ResourceTemplatesTypeItem item = null;
			ComputingResourceTemplate crt = null;
			
			// loop iterates over all templates
			for(int i = 0; i < this.resourceTemplates.getResourceTemplatesTypeItemCount() && !found; i++){
				item = this.resourceTemplates.getResourceTemplatesTypeItem(i);
				crt = item.getComputingResourceTemplate();
				
				// looking for correct template
				if(refId.equals(crt.getTemplateId())){
				
					ComputingResourceTemplateItem crtItem = null;
					
					// looking for correct parameter in template
					for(int j = 0; j < crt.getComputingResourceTemplateItemCount() && !found; j++){
						crtItem = crt.getComputingResourceTemplateItem(j);
						
						if((netParam = crtItem.getNetworkParameter()) != null){
							
							if(netParam.getName().equals(paramName)){
								found = true;
							}
						}
					}
				}
			}
			
		} else if((choice = computingResource.getComputingResourceBaseTypeChoice()) != null) {
		
			for(int i = 0; i < choice.getComputingResourceBaseTypeChoiceItemCount() && !found; i++){
				crItem = choice.getComputingResourceBaseTypeChoiceItem(i);
				
				if((netParam = crItem.getNetworkParameter()) != null){
					
					if(netParam.getName().equals(paramName)){
						found = true;
					}
				}
			}
		}
		
		if(!found)
			throw new NoSuchParamException(paramName, "Parameter is not defined.");
		
		return netParam;
	}
	
	public OtherParameterType getOtherParam(String paramName) throws NoSuchParamException{
		if(computingResource == null)
			throw new NoSuchParamException("Computing resource is unavailable. " +
											"Please set computing resource before exploring its values.");
		
		boolean found = false;
		ComputingResourceBaseTypeChoiceItem crItem = null;
		OtherParameterType otherParam = null;
		String refId = null;
		ComputingResourceBaseTypeChoice choice = null;
		
		
		if((refId = computingResource.getTemplateIdReference()) != null){
			if(this.resourceTemplates == null)
				throw new NoSuchParamException(paramName, "Computing resource template is not defined.");
			
			ResourceTemplatesTypeItem item = null;
			ComputingResourceTemplate crt = null;
			
			// loop iterates over all templates
			for(int i = 0; i < this.resourceTemplates.getResourceTemplatesTypeItemCount() && !found; i++){
				item = this.resourceTemplates.getResourceTemplatesTypeItem(i);
				crt = item.getComputingResourceTemplate();
				
				// looking for correct template
				if(refId.equals(crt.getTemplateId())){
				
					ComputingResourceTemplateItem crtItem = null;
					
					// looking for correct parameter in template
					for(int j = 0; j < crt.getComputingResourceTemplateItemCount() && !found; j++){
						crtItem = crt.getComputingResourceTemplateItem(j);
						
						if((otherParam = crtItem.getOtherParameter()) != null){
							
							if(otherParam.getName().equals(paramName)){
								found = true;
							}
						}
					}
				}
			}
			
		} else if((choice = computingResource.getComputingResourceBaseTypeChoice()) != null){
		
			for(int i = 0; i < choice.getComputingResourceBaseTypeChoiceItemCount() && !found; i++){
				crItem = choice.getComputingResourceBaseTypeChoiceItem(i);
				
				if((otherParam = crItem.getOtherParameter()) != null) {
					
					if(otherParam.getName().equals(paramName)){
						found = true;
					}
				}
					
			}
		}
		if(!found)
			throw new NoSuchParamException(paramName, "Parameter " + paramName + " is not defined.");
		
		return otherParam;
	}
	
	public String[] getApplicationName() throws NoSuchParamException {
		return getParamStringValue(ComputingParameterName.APPLICATION);
	}

	public String[] getCpuarch() throws NoSuchParamException {
		return getParamStringValue(ComputingParameterName.CPUARCH);
	}

	public double[] getCpucount() throws NoSuchParamException {
		return getParamDoubleValue(ComputingParameterName.CPUCOUNT);
	}

	public double[] getCpuLoad() throws NoSuchParamException {
		throw new NoSuchParamException("cpuload");
	}

	public double[] getCpuspeed() throws NoSuchParamException {
		return getParamDoubleValue(ComputingParameterName.CPUSPEED);
	}

	public double[] getDiskspace() throws NoSuchParamException {
		return getParamDoubleValue(ComputingParameterName.DISKSPACE);
	}

	public double[] getFreecpus() throws NoSuchParamException {
		return getParamDoubleValue(ComputingParameterName.FREECPUS);
	}

	public double[] getFreediskspace() throws NoSuchParamException {
		return getParamDoubleValue(ComputingParameterName.FREEDISKSPACE);
	}

	public double[] getFreememory() throws NoSuchParamException {
		return getParamDoubleValue(ComputingParameterName.FREEMEMORY);
	}

	public String[] getHostname() throws NoSuchParamException {
		return getParamStringValue(ComputingParameterName.HOSTNAME);
	}

	public String[] getLocalResourceManager() throws NoSuchParamException {
		return getParamStringValue(ComputingParameterName.LOCALRESOURCEMANAGER);
	}

	public double[] getMemory() throws NoSuchParamException {
		return getParamDoubleValue(ComputingParameterName.MEMORY);
	}

	public String[] getOsname() throws NoSuchParamException {
		return getParamStringValue(ComputingParameterName.OSNAME);
	}

	public String[] getOsrelease() throws NoSuchParamException {
		return getParamStringValue(ComputingParameterName.OSRELEASE);
	}

	public String[] getOstype() throws NoSuchParamException {
		return getParamStringValue(ComputingParameterName.OSTYPE);
	}

	public String[] getOsversion() throws NoSuchParamException {
		return getParamStringValue(ComputingParameterName.OSVERSION);
	}

	public String[] getRemoteSubmissionInterface() throws NoSuchParamException {
		return getParamStringValue(ComputingParameterName.REMOTESUBMISSIONINTERFACE);
	}
	public double[] getBandwidth() throws NoSuchParamException {
		return getParamDoubleValue(NetworkParameterName.BANDWIDTH);
	}
	
	public double[] getLatency() throws NoSuchParamException {
		return getParamDoubleValue(NetworkParameterName.LATENCY);
	}
		
	public double[] getCapacity() throws NoSuchParamException {
		return getParamDoubleValue(NetworkParameterName.CAPACITY);
	}	

	public void explore(ComputingResourceType computingResource) {
		this.computingResource = computingResource;
	}

	public double getMaxBandwidth() throws NoSuchParamException {
		return getParamMaxValue(NetworkParameterName.BANDWIDTH);
	}

	public double getMaxCapacity() throws NoSuchParamException {
		return getParamMaxValue(NetworkParameterName.CAPACITY);
	}

	public double getMaxCpucount() throws NoSuchParamException {
		return getParamMaxValue(ComputingParameterName.CPUCOUNT);
	}

	public double getMaxCpuload() throws NoSuchParamException {
		throw new NoSuchParamException("cpuload");
	}

	public double getMaxCpuspeed() throws NoSuchParamException {
		return getParamMaxValue(ComputingParameterName.CPUSPEED);
	}

	public double getMaxDiskspace() throws NoSuchParamException {
		return getParamMaxValue(ComputingParameterName.DISKSPACE);
	}

	public double getMaxFreecpus() throws NoSuchParamException {
		return getParamMaxValue(ComputingParameterName.FREECPUS);
	}

	public double getMaxFreediskspace() throws NoSuchParamException {
		return getParamMaxValue(ComputingParameterName.FREEDISKSPACE);
	}

	public double getMaxFreememory() throws NoSuchParamException {
		return getParamMaxValue(ComputingParameterName.FREEMEMORY);
	}

	public double getMaxLatency() throws NoSuchParamException {
		return getParamMaxValue(NetworkParameterName.LATENCY);
	}

	public double getMaxMemory() throws NoSuchParamException {
		return getParamMaxValue(ComputingParameterName.MEMORY);
	}

	public double getMinBandwidth() throws NoSuchParamException {
		return getParamMinValue(NetworkParameterName.BANDWIDTH);
	}

	public double getMinCapacity() throws NoSuchParamException {
		return getParamMinValue(NetworkParameterName.CAPACITY);
	}

	public double getMinCpucount() throws NoSuchParamException {
		return getParamMinValue(ComputingParameterName.CPUCOUNT);
	}

	public double getMinCpuload() throws NoSuchParamException {
		throw new NoSuchParamException("cpuload");
	}

	public double getMinCpuspeed() throws NoSuchParamException {
		return getParamMinValue(ComputingParameterName.CPUSPEED);
	}

	public double getMinDiskspace() throws NoSuchParamException {
		return getParamMinValue(ComputingParameterName.DISKSPACE);
	}

	public double getMinFreecpus() throws NoSuchParamException {
		return getParamMinValue(ComputingParameterName.FREECPUS);
	}

	public double getMinFreediskspace() throws NoSuchParamException {
		return getParamMinValue(ComputingParameterName.FREEDISKSPACE);
	}

	public double getMinFreememory() throws NoSuchParamException {
		return getParamMinValue(ComputingParameterName.FREEMEMORY);
	}

	public double getMinLatency() throws NoSuchParamException {
		return getParamMinValue(NetworkParameterName.LATENCY);
	}

	public double getMinMemory() throws NoSuchParamException {
		return getParamMinValue(ComputingParameterName.MEMORY);
	}

	public double[][] getRangeBandwidth() throws NoSuchParamException {
		return getParamRangeValue(NetworkParameterName.BANDWIDTH);
	}

	public double[][] getRangeCapacity() throws NoSuchParamException {
		return getParamRangeValue(NetworkParameterName.CAPACITY);
	}

	public double[][] getRangeCpucount() throws NoSuchParamException {
		return getParamRangeValue(ComputingParameterName.CPUCOUNT);
	}

	public double[][] getRangeCpuload() throws NoSuchParamException {
		throw new NoSuchParamException("cpuload");
	}

	public double[][] getRangeCpuspeed() throws NoSuchParamException {
		return getParamRangeValue(ComputingParameterName.CPUSPEED);
	}

	public double[][] getRangeDiskspace() throws NoSuchParamException {
		return getParamRangeValue(ComputingParameterName.DISKSPACE);
	}

	public double[][] getRangeFreecpus() throws NoSuchParamException {
		return getParamRangeValue(ComputingParameterName.FREECPUS);
	}

	public double[][] getRangeFreediskspace() throws NoSuchParamException {
		return getParamRangeValue(ComputingParameterName.FREEDISKSPACE);
	}

	public double[][] getRangeFreememory() throws NoSuchParamException {
		return getParamRangeValue(ComputingParameterName.FREEMEMORY);
	}

	public double[][] getRangeLatency() throws NoSuchParamException {
		return getParamRangeValue(NetworkParameterName.LATENCY);
	}

	public double[][] getRangeMemory() throws NoSuchParamException {
		return getParamRangeValue(ComputingParameterName.MEMORY);
	}
	
	
	
	protected double getParamMinValue(ComputingParameterName paramName) throws NoSuchParamException{
		ComputingResourceParameterType hostParam = null;
		ParameterBoundaryType pbt = null;
		
		hostParam = getHostParam(paramName);
		pbt = hostParam.getMin();
		if(pbt == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		return pbt.getContent();
	}
	
	
	protected double getParamMaxValue(ComputingParameterName paramName) throws NoSuchParamException{
		ComputingResourceParameterType hostParam = null;
		ParameterBoundaryType pbt = null;
		
		hostParam = getHostParam(paramName);
		pbt = hostParam.getMax();
		if(pbt == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		return pbt.getContent();
	}
	
	
	protected double[] getParamDoubleValue(ComputingParameterName paramName) throws NoSuchParamException{
		double value[] = null;
		ArrayList<Double> list = null;
		ComputingResourceParameterType hostParam = null;
		ParameterTypeChoice choice = null;
		ParameterTypeChoiceItem choiceItem = null;
		Value paramValue = null;
		
		hostParam = getHostParam(paramName);
		choice = hostParam.getParameterTypeChoice();

		if(choice == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");

		list = new ArrayList<Double>();
		for(int i = 0; i < choice.getParameterTypeChoiceItemCount(); i++){
			choiceItem = choice.getParameterTypeChoiceItem(i);
			paramValue = choiceItem.getValue();
			if(paramValue == null) continue;
			
			list.add(paramValue.getContent());
			
		}
		
		value = new double[list.size()];

		for(int i = 0; i < value.length; i++){
			value[i] = list.get(i);
		}

		return value;
	}
	
	protected String[] getParamStringValue(ComputingParameterName paramName) throws NoSuchParamException{
		String value[] = null;
		StringValue stringValue[] = null;
		ComputingResourceParameterType hostParam = null;
		
		hostParam = getHostParam(paramName);
		stringValue = hostParam.getStringValue();
		if(stringValue == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		value = new String[stringValue.length];

		for(int i = 0; i < value.length; i++){
			value[i] = stringValue[i].getValue();
		}
		
		return value;
	}
	
	/**
	 * 
	 * @param paramName
	 * @return 2dim tale, tab[i][0] - min value, tab[i][1] - max value
	 * @throws NoSuchParamException
	 */
	protected double[][] getParamRangeValue(ComputingParameterName paramName) throws NoSuchParamException{
		double value[][] = null;
		ArrayList<Double> list = null;
		ComputingResourceParameterType hostParam = null;
		ParameterTypeChoice choice = null;
		ParameterTypeChoiceItem choiceItem = null;
		Range paramValue = null;
		
		hostParam = getHostParam(paramName);
		choice = hostParam.getParameterTypeChoice();

		if(choice == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");

		list = new ArrayList<Double>();
		for(int i = 0; i < choice.getParameterTypeChoiceItemCount(); i++){
			choiceItem = choice.getParameterTypeChoiceItem(i);
			paramValue = choiceItem.getRange();
			if(paramValue == null) continue;
			
			list.add(paramValue.getMin().getContent());
			list.add(paramValue.getMax().getContent());
			
		}
		
		value = new double[list.size() / 2][2];
		int j = 0;
		for(int i = 0; i < list.size(); i += 2, j++){
			value[j][0] = list.get(i);	 // min value
			value[j][1] = list.get(i+1); // max value
		}
		
		return value;
	}
	
	
	
	
	protected double getParamMinValue(NetworkParameterName paramName) throws NoSuchParamException{
		ComputingResourceNetParameterType hostParam = null;
		ParameterBoundaryType pbt = null;
		
		hostParam = getNetParam(paramName);
		pbt = hostParam.getMin();
		if(pbt == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		return pbt.getContent();
	}
	
	
	protected double getParamMaxValue(NetworkParameterName paramName) throws NoSuchParamException{
		ComputingResourceNetParameterType hostParam = null;
		ParameterBoundaryType pbt = null;
		
		hostParam = getNetParam(paramName);
		pbt = hostParam.getMax();
		if(pbt == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		return pbt.getContent();
	}
	
	
	protected double[] getParamDoubleValue(NetworkParameterName paramName) throws NoSuchParamException{
		double value[] = null;
		ArrayList<Double> list = null;
		ComputingResourceNetParameterType hostParam = null;
		ParameterTypeChoice choice = null;
		ParameterTypeChoiceItem choiceItem = null;
		Value paramValue = null;
		
		hostParam = getNetParam(paramName);
		choice = hostParam.getParameterTypeChoice();

		if(choice == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");

		list = new ArrayList<Double>();
		for(int i = 0; i < choice.getParameterTypeChoiceItemCount(); i++){
			choiceItem = choice.getParameterTypeChoiceItem(i);
			paramValue = choiceItem.getValue();
			if(paramValue == null) continue;
			
			list.add(paramValue.getContent());
			
		}
		
		value = new double[list.size()];

		for(int i = 0; i < value.length; i++){
			value[i] = list.get(i);
		}

		return value;
	}
	
	protected String[] getParamStringValue(NetworkParameterName paramName) throws NoSuchParamException{
		String value[] = null;
		StringValue stringValue[] = null;
		ComputingResourceNetParameterType hostParam = null;
		
		hostParam = getNetParam(paramName);
		stringValue = hostParam.getStringValue();
		if(stringValue == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		value = new String[stringValue.length];

		for(int i = 0; i < value.length; i++){
			value[i] = stringValue[i].getValue();
		}
		
		return value;
	}
	
	/**
	 * 
	 * @param paramName
	 * @return 2dim tale, tab[i][0] - min value, tab[i][1] - max value
	 * @throws NoSuchParamException
	 */
	protected double[][] getParamRangeValue(NetworkParameterName paramName) throws NoSuchParamException{
		double value[][] = null;
		ArrayList<Double> list = null;
		ComputingResourceNetParameterType hostParam = null;
		ParameterTypeChoice choice = null;
		ParameterTypeChoiceItem choiceItem = null;
		Range paramValue = null;
		
		hostParam = getNetParam(paramName);
		choice = hostParam.getParameterTypeChoice();

		if(choice == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");

		list = new ArrayList<Double>();
		for(int i = 0; i < choice.getParameterTypeChoiceItemCount(); i++){
			choiceItem = choice.getParameterTypeChoiceItem(i);
			paramValue = choiceItem.getRange();
			if(paramValue == null) continue;
			
			list.add(paramValue.getMin().getContent());
			list.add(paramValue.getMax().getContent());
			
		}
		
		value = new double[list.size() / 2][2];
		int j = 0;
		for(int i = 0; i < list.size(); i += 2, j++){
			value[j][0] = list.get(i);	 // min value
			value[j][1] = list.get(i+1); // max value
		}
		
		return value;
	}
	
	public void setResourceTemplates(ResourceTemplatesType resourceTemplates){
		this.resourceTemplates = resourceTemplates;
	}

}
