package org.qcg.broker.schemas.jobdesc.wrapper.impl;

import org.qcg.broker.schemas.exception.ElementNotSpecified;
import org.qcg.broker.schemas.exception.ValueNotSpecified;
import org.qcg.broker.schemas.jobdesc.Parameter;
import org.qcg.broker.schemas.jobdesc.ParameterBoundaryType;
import org.qcg.broker.schemas.jobdesc.ParameterChoice;
import org.qcg.broker.schemas.jobdesc.ParameterValueType;
import org.qcg.broker.schemas.jobdesc.PreferencesType;
import org.qcg.broker.schemas.jobdesc.RangeType;
import org.qcg.broker.schemas.jobdesc.RequirementsType;
import org.qcg.broker.schemas.jobdesc.ResourceRequirementsType;
import org.qcg.broker.schemas.jobdesc.Task;
import org.qcg.broker.schemas.jobdesc.wrapper.TaskPreferences;

/**
 * 
 * @author Marcin Krystek
 *
 */
public class TaskPreferencesImpl implements TaskPreferences{

	
	protected Task task = null;
	
	public TaskPreferencesImpl(){
		this.task = null;
	}
	
	public TaskPreferencesImpl(Task task){
		this.task = task;
	}
	
	public void wrap(Task task) {
		this.task = task;
	}
	
	public double getMaxValue(Parameter parameter) throws ValueNotSpecified {
		if(parameter == null)
			throw new ValueNotSpecified();
		
		ParameterChoice choice = null;
		ParameterBoundaryType value = null;
		
		choice = parameter.getParameterChoice();
		if(choice == null)
			throw new ValueNotSpecified();
		
		value = choice.getMax();
		if(value == null)
			throw new ValueNotSpecified();
		
		return value.getContent();
	}

	public double getMinValue(Parameter parameter) throws ValueNotSpecified {
		if(parameter == null)
			throw new ValueNotSpecified();
		
		ParameterChoice choice = null;
		ParameterBoundaryType value = null;
		
		choice = parameter.getParameterChoice();
		if(choice == null)
			throw new ValueNotSpecified();
		
		value = choice.getMin();
		if(value == null)
			throw new ValueNotSpecified();
		
		return value.getContent();
	}

	public double[] getRangeValue(Parameter parameter) throws ValueNotSpecified {
		if(parameter == null)
			throw new ValueNotSpecified();
		
		ParameterChoice choice = null;
		RangeType value = null;
		double tab[] = null;
		
		choice = parameter.getParameterChoice();
		if(choice == null)
			throw new ValueNotSpecified();
		
		value = choice.getRange();
		if(value == null)
			throw new ValueNotSpecified();
		
		tab = new double[2];
		tab[0] = value.getMin().getContent();
		tab[1] = value.getMax().getContent();
		
		return tab;
	}

	public String getStringValue(Parameter parameter) throws ValueNotSpecified {
		if(parameter == null)
			throw new ValueNotSpecified();
		
		String value = null;
		
		value = parameter.getStringValue();

		if(value == null)
			throw new ValueNotSpecified();
		
		return value;
	}

	public double getValue(Parameter parameter) throws ValueNotSpecified {
		if(parameter == null)
			throw new ValueNotSpecified();
		
		ParameterChoice choice = null;
		ParameterValueType value = null;
		
		choice = parameter.getParameterChoice();
		if(choice == null)
			throw new ValueNotSpecified();
		
		value = choice.getValue();
		if(value == null)
			throw new ValueNotSpecified();
		
		return value.getContent();
	}

	
	public PreferencesType getPreferences() throws ElementNotSpecified {
		RequirementsType requirements = null;
		ResourceRequirementsType trr = null;
		PreferencesType pref = null;
		
		if(task == null) 
			throw new ElementNotSpecified("Task is not specified.");
		
		requirements = task.getRequirements();
		if(requirements == null)
			throw new ElementNotSpecified("Requirements not specified for this task.");
		
		trr = requirements.getResourceRequirements();
		if(trr == null)
			throw new ElementNotSpecified("Resource requirements not specified for this task.");
		
		pref = trr.getPreferences();
		if(pref == null)
			throw new ElementNotSpecified("Preferences not specified for this task.");
		
		return pref;
	}

	public Parameter getParameter(String name) throws ElementNotSpecified {
		PreferencesType prefs = null;
		Parameter param = null;
		
		prefs = getPreferences();
		for(int i = 0; i < prefs.getParameterCount(); i++){
			param = prefs.getParameter(i);
			if(param.getName().equals(name))
				break;
		}
		if(param == null)
			throw new ElementNotSpecified(name + " parameter not defined for this task.");
		
		return param;
	}
}
