package org.qcg.broker.schemas.jobdesc.wrapper.impl;

import java.util.ArrayList;

import org.qcg.broker.schemas.exception.NoSuchParamException;
import org.qcg.broker.schemas.jobdesc.ComputingResourceNetParameterType;
import org.qcg.broker.schemas.jobdesc.ComputingResourceParameterType;
import org.qcg.broker.schemas.jobdesc.ComputingResourceType;
import org.qcg.broker.schemas.jobdesc.OtherParameterType;
import org.qcg.broker.schemas.jobdesc.RequirementsType;
import org.qcg.broker.schemas.jobdesc.ResourceRequirementsType;
import org.qcg.broker.schemas.jobdesc.Task;
import org.qcg.broker.schemas.jobdesc.types.ComputingParameterName;
import org.qcg.broker.schemas.jobdesc.types.NetworkParameterName;
import org.qcg.broker.schemas.jobdesc.wrapper.ComputingResourceJobDescParamExplorer;
import org.qcg.broker.schemas.jobdesc.wrapper.TaskRequirements;
import org.qcg.broker.schemas.jobdesc.wrapper.impl.ComputingResourceParamExplorerImpl;
import org.qcg.broker.schemas.utils.ComputingResourceParamExplorer;




/**
 * 
 * @author Marcin Krystek
 *
 */

public class TaskRequirementsImpl implements TaskRequirements{

	protected Task task;
	
	protected enum ValueType{
		DOUBLE_VALUE,
		MIN_VALUE,
		MAX_VALUE;
	};
	
	public TaskRequirementsImpl(){
		this.task = null;
	}
	
	public TaskRequirementsImpl(Task task){
		this.task = task;
	}
	
	public void wrap(Task task){
		this.task = task;
	}
	
	public String[] getApplication() throws NoSuchParamException {
		return getStringValues(ComputingParameterName.APPLICATION);
	}

	public double[] getBandwidth() throws NoSuchParamException {
		return getDoubleValues(NetworkParameterName.BANDWIDTH, ValueType.DOUBLE_VALUE);
	}

	public double[] getCapacity() throws NoSuchParamException {
		return getDoubleValues(NetworkParameterName.CAPACITY, ValueType.DOUBLE_VALUE);
	}

	public String[] getCpuarch() throws NoSuchParamException {
		return getStringValues(ComputingParameterName.CPUARCH);
	}

	public double[] getCpucount() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.CPUCOUNT, ValueType.DOUBLE_VALUE);
	}

	public double[] getCpuspeed() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.CPUSPEED, ValueType.DOUBLE_VALUE);
	}

	public double[] getDiskspace() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.DISKSPACE, ValueType.DOUBLE_VALUE);
	}

	public double[] getFreecpus() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.FREECPUS, ValueType.DOUBLE_VALUE);
	}

	public double[] getFreediskspace() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.FREEDISKSPACE, ValueType.DOUBLE_VALUE);
	}

	public double[] getFreememory() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.FREEMEMORY, ValueType.DOUBLE_VALUE);
	}

	public String[] getHostname() throws NoSuchParamException {
		return getStringValues(ComputingParameterName.HOSTNAME);
	}

	public double[] getLatency() throws NoSuchParamException {
		return getDoubleValues(NetworkParameterName.LATENCY, ValueType.DOUBLE_VALUE);
	}

	public String[] getLocalResourceManager() throws NoSuchParamException {
		return getStringValues(ComputingParameterName.LOCALRESOURCEMANAGER);
	}

	public double[] getMemory() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.MEMORY, ValueType.DOUBLE_VALUE);
	}
	
	public double[] getMaxBandwidth() throws NoSuchParamException {
		return getDoubleValues(NetworkParameterName.BANDWIDTH, ValueType.MAX_VALUE);
	}

	public double[] getMaxCapacity() throws NoSuchParamException {
		return getDoubleValues(NetworkParameterName.BANDWIDTH, ValueType.MAX_VALUE);
	}

	public double[] getMaxCpucount() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.CPUCOUNT, ValueType.MAX_VALUE);
	}

	public double[] getMaxCpuspeed() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.CPUSPEED, ValueType.MAX_VALUE);
	}

	public double[] getMaxDiskspace() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.DISKSPACE, ValueType.MAX_VALUE);
	}

	public double[] getMaxFreecpus() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.FREECPUS, ValueType.MAX_VALUE);
	}

	public double[] getMaxFreediskspace() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.FREEDISKSPACE, ValueType.MAX_VALUE);
	}

	public double[] getMaxFreememory() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.FREEMEMORY, ValueType.MAX_VALUE);
	}

	public double[] getMaxLatency() throws NoSuchParamException {
		return getDoubleValues(NetworkParameterName.LATENCY, ValueType.MAX_VALUE);
	}

	public double[] getMaxMemory() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.MEMORY, ValueType.MAX_VALUE);
	}


	public double[] getMinBandwidth() throws NoSuchParamException {
		return getDoubleValues(NetworkParameterName.BANDWIDTH, ValueType.MIN_VALUE);
	}

	public double[] getMinCapacity() throws NoSuchParamException {
		return getDoubleValues(NetworkParameterName.CAPACITY, ValueType.MIN_VALUE);
	}

	public double[] getMinCpucount() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.CPUCOUNT, ValueType.MIN_VALUE);
	}

	public double[] getMinCpuspeed() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.CPUSPEED, ValueType.MIN_VALUE);
	}

	public double[] getMinDiskspace() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.DISKSPACE, ValueType.MIN_VALUE);
	}

	public double[] getMinFreecpus() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.FREECPUS, ValueType.MIN_VALUE);
	}

	public double[] getMinFreediskspace() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.FREEDISKSPACE, ValueType.MIN_VALUE);
	}

	public double[] getMinFreememory() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.FREEMEMORY, ValueType.MIN_VALUE);
	}

	public double[] getMinLatency() throws NoSuchParamException {
		return getDoubleValues(NetworkParameterName.LATENCY, ValueType.MIN_VALUE);
	}

	public double[] getMinMemory() throws NoSuchParamException {
		return getDoubleValues(ComputingParameterName.MEMORY, ValueType.MIN_VALUE);
	}
	
	
	public double[][] getRangFreememory() throws NoSuchParamException {
		return getRangeValues(ComputingParameterName.FREEMEMORY);
	}

	public double[][] getRangeBandwidth() throws NoSuchParamException {
		return getRangeValues(NetworkParameterName.BANDWIDTH);
	}

	public double[][] getRangeCapacity() throws NoSuchParamException {
		return getRangeValues(NetworkParameterName.CAPACITY);
	}

	public double[][] getRangeCpucount() throws NoSuchParamException {
		return getRangeValues(ComputingParameterName.CPUCOUNT);
	}

	public double[][] getRangeCpuspeed() throws NoSuchParamException {
		return getRangeValues(ComputingParameterName.CPUSPEED);
	}

	public double[][] getRangeDiskspace() throws NoSuchParamException {
		return getRangeValues(ComputingParameterName.DISKSPACE);
	}

	public double[][] getRangeFreecpus() throws NoSuchParamException {
		return getRangeValues(ComputingParameterName.FREECPUS);
	}

	public double[][] getRangeFreediskspace() throws NoSuchParamException {
		return getRangeValues(ComputingParameterName.FREEDISKSPACE);
	}

	public double[][] getRangeLatency() throws NoSuchParamException {
		return getRangeValues(NetworkParameterName.LATENCY);
	}

	public double[][] getRangeMemory() throws NoSuchParamException {
		return getRangeValues(ComputingParameterName.MEMORY);
	}
	

	public String[] getOsname() throws NoSuchParamException {
		return getStringValues(ComputingParameterName.OSNAME);
	}

	public String[] getOsrelease() throws NoSuchParamException {
		return getStringValues(ComputingParameterName.OSRELEASE);
	}

	public String[] getOstype() throws NoSuchParamException {
		return getStringValues(ComputingParameterName.OSTYPE);
	}

	public String[] getOsversion() throws NoSuchParamException {
		return getStringValues(ComputingParameterName.OSVERSION);
	}

	public String[] getRemoteSubmissionInterface() throws NoSuchParamException {
		return getStringValues(ComputingParameterName.REMOTESUBMISSIONINTERFACE);
	}
	
	protected String[] getStringValues(ComputingParameterName param) throws NoSuchParamException{
		RequirementsType req = null;
		ResourceRequirementsType taskResReq = null;
		ComputingResourceType compRes = null;
		String values[] = null;
		ArrayList<String> list = null;
		ComputingResourceParamExplorer
					<ComputingResourceType, ComputingResourceNetParameterType, ComputingResourceParameterType, 
					OtherParameterType, ComputingParameterName, NetworkParameterName> 
						explorer = null;
		
		req = task.getRequirements();
		if(req == null)
			throw new NoSuchParamException("Requirements are not defined.");
		
		taskResReq = req.getResourceRequirements();
		if(taskResReq == null)
			throw new NoSuchParamException("Resource requirements are not defined.");
		
		list = new ArrayList<String>();
		explorer = new ComputingResourceParamExplorerImpl();
		
		for(int i = 0; i < taskResReq.getComputingResourceCount(); i++){
			compRes = taskResReq.getComputingResource(i);
			explorer.explore(compRes);
			
			values = null;
			try {
				switch(param){
					case OSNAME: values = explorer.getOsname(); 
						break;
					case OSTYPE: values = explorer.getOstype();
						break;
					case CPUARCH: values = explorer.getCpuarch();
						break;
					case OSVERSION: values = explorer.getOsversion();
						break;
					case OSRELEASE: values = explorer.getOsrelease();
						break;
					case APPLICATION: values = explorer.getApplicationName();
						break;
					case REMOTESUBMISSIONINTERFACE:
									values = explorer.getRemoteSubmissionInterface();
						break;
					case LOCALRESOURCEMANAGER: 
									values = explorer.getLocalResourceManager();
						break;
					case HOSTNAME:
									values = explorer.getHostname();
						break;
				}
			} catch (NoSuchParamException e){
				values = null;
			}
			
			if(values != null)
				list.add(values[0]);
			
		}
		
		return list.toArray(new String[0]);
	}

	protected double[] getDoubleValues(ComputingParameterName param, ValueType valueType) throws NoSuchParamException{
		RequirementsType req = null;
		ResourceRequirementsType taskResReq = null;
		ComputingResourceType compRes = null;
		double values[] = null;
		double value = -1;
		
		ArrayList<Double> list = null;
		ComputingResourceJobDescParamExplorer explorer = null;
		
		req = task.getRequirements();
		if(req == null)
			throw new NoSuchParamException("Requirements are not defined.");
		
		taskResReq = req.getResourceRequirements();
		if(taskResReq == null)
			throw new NoSuchParamException("Resource requirements are not defined.");
		
		list = new ArrayList<Double>();
		explorer = new ComputingResourceParamExplorerImpl();
		explorer.setResourceTemplates(req.getResourceTemplates());
		
		
		for(int i = 0; i < taskResReq.getComputingResourceCount(); i++){
			compRes = taskResReq.getComputingResource(i);
			explorer.explore(compRes);
			
			values = null;
			value = -1;
			try {
				switch(param){
					
				case MEMORY: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getMemory();
								break;
							case MIN_VALUE: value = explorer.getMinMemory();
								break;
							case MAX_VALUE: value = explorer.getMaxMemory();
								break;
						}
						 
						break;
					
					case FREEMEMORY: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getFreememory();
								break;
							case MIN_VALUE: value = explorer.getMinFreememory();
								break;
							case MAX_VALUE: value = explorer.getMaxFreememory();
								break;
						}
						
						break;
						
					case CPUCOUNT: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getCpucount();
								break;
							case MIN_VALUE: value = explorer.getMinCpucount(); 
								break;
							case MAX_VALUE: value = explorer.getMaxCpucount();
								break;
						}
						 
						break;
						
					case FREECPUS: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getFreecpus();
								break;
							case MIN_VALUE: value = explorer.getMinFreecpus();
								break;
							case MAX_VALUE: value = explorer.getMaxFreecpus();
								break;
						}
						
						break;

					case CPUSPEED: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getCpuspeed();
								break;
							case MIN_VALUE: value = explorer.getMinCpuspeed();
								break;
							case MAX_VALUE: value = explorer.getMaxCpuspeed();
								break;
						}
						
						break;
						
					case DISKSPACE: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getDiskspace();
								break;
							case MIN_VALUE: value = explorer.getMinDiskspace(); 
								break;
							case MAX_VALUE: value = explorer.getMaxDiskspace();
								break;
						}
						 
						break;

					case FREEDISKSPACE: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getFreediskspace();
								break;
							case MIN_VALUE: value = explorer.getMinFreediskspace();
								break;
							case MAX_VALUE: value = explorer.getMaxFreediskspace(); 
								break;
						}
						
						break;
				}
			} catch (NoSuchParamException e){
				values = null;
				value = -1;
			}
			
			if(valueType == ValueType.DOUBLE_VALUE && values != null && values.length > 0){
				list.add(values[0]);
			} else if(valueType != ValueType.DOUBLE_VALUE && value != -1){
				list.add(value);
			}
			
		}
		
		values = new double[list.size()];
		for(int i = 0; i < list.size(); i++){
			values[i] = list.get(i);
		}
		
		return values;
	}
	
	protected double[] getDoubleValues(NetworkParameterName param, ValueType valueType) throws NoSuchParamException{
		RequirementsType req = null;
		ResourceRequirementsType taskResReq = null;
		ComputingResourceType compRes = null;
		double values[] = null;
		double value = -1;
		ArrayList<Double> list = null;
		ComputingResourceJobDescParamExplorer explorer = null;
		
		req = task.getRequirements();
		if(req == null)
			throw new NoSuchParamException("Requirements are not defined.");
		
		taskResReq = req.getResourceRequirements();
		if(taskResReq == null)
			throw new NoSuchParamException("Resource requirements are not defined.");
		
		list = new ArrayList<Double>();
		explorer = new ComputingResourceParamExplorerImpl();
		
		for(int i = 0; i < taskResReq.getComputingResourceCount(); i++){
			compRes = taskResReq.getComputingResource(i);
			explorer.explore(compRes);
			
			values = null;
			value = -1;
			try {
				switch(param){
					case BANDWIDTH: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getBandwidth();
								break;
							case MIN_VALUE: value = explorer.getMinBandwidth(); 
								break;
							case MAX_VALUE: value = explorer.getMaxBandwidth();
								break;
						}
						break;

					case CAPACITY: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getCapacity();
								break;
							case MIN_VALUE: value = explorer.getMinCapacity(); 
								break;
							case MAX_VALUE: value = explorer.getMaxCapacity();
								break;
						}
						break;

					case LATENCY: 
						switch(valueType) {
							case DOUBLE_VALUE: values = explorer.getLatency();
								break;
							case MIN_VALUE: value = explorer.getMinLatency(); 
								break;
							case MAX_VALUE: value = explorer.getMaxLatency();
								break;
						}
						break;
				}
			} catch (NoSuchParamException e){
				values = null;
				value = -1;
			}
			
			
			if(valueType == ValueType.DOUBLE_VALUE && values != null && values.length > 0) {
				list.add(values[0]);
			} else if(valueType != ValueType.DOUBLE_VALUE && value != -1){
				list.add(value);
			}
			
		}
		
		values = new double[list.size()];
		for(int i = 0; i < list.size(); i++){
			values[i] = list.get(i);
		}
		
		return values;
	}
	
	
	
	protected double[][] getRangeValues(ComputingParameterName param) throws NoSuchParamException{
		RequirementsType req = null;
		ResourceRequirementsType taskResReq = null;
		ComputingResourceType compRes = null;
		double values[][] = null;
		
		ArrayList<double[]> list = null;
		ComputingResourceJobDescParamExplorer explorer = null;
		
		req = task.getRequirements();
		if(req == null)
			throw new NoSuchParamException("Requirements are not defined.");
		
		taskResReq = req.getResourceRequirements();
		if(taskResReq == null)
			throw new NoSuchParamException("Resource requirements are not defined.");
		
		list = new ArrayList<double[]>();
		explorer = new ComputingResourceParamExplorerImpl();
		
		for(int i = 0; i < taskResReq.getComputingResourceCount(); i++){
			compRes = taskResReq.getComputingResource(i);
			explorer.explore(compRes);
			
			values = null;
			try {
				switch(param){
					case MEMORY: values = explorer.getRangeMemory();
						break;
					case FREEMEMORY: values = explorer.getRangeFreememory();
						break;
					case CPUCOUNT: values = explorer.getRangeCpucount();
						break;
					case FREECPUS: values = explorer.getRangeFreecpus();
						break;
					case CPUSPEED: values = explorer.getRangeCpuspeed();
						break;
					case DISKSPACE: values = explorer.getRangeDiskspace();
						break;
					case FREEDISKSPACE: values = explorer.getRangeFreediskspace();
						break;
				}
			} catch (NoSuchParamException e){
				values = null;
			}
			
			if(values != null && values.length > 0){
				list.add(values[0]);
			} 
			
		}
		
		values = new double[list.size()][];

		for(int i = 0; i < list.size(); i++){
			values[i] = list.get(i); 
		}
		
		return values;
	}
	
	protected double[][] getRangeValues(NetworkParameterName param) throws NoSuchParamException{
		RequirementsType req = null;
		ResourceRequirementsType taskResReq = null;
		ComputingResourceType compRes = null;
		double values[][] = null;
		ArrayList<double[]> list = null;
		ComputingResourceJobDescParamExplorer explorer = null;
		
		req = task.getRequirements();
		if(req == null)
			throw new NoSuchParamException("Requirements are not defined.");
		
		taskResReq = req.getResourceRequirements();
		if(taskResReq == null)
			throw new NoSuchParamException("Resource requirements are not defined.");
		
		list = new ArrayList<double[]>();
		explorer = new ComputingResourceParamExplorerImpl();
		
		for(int i = 0; i < taskResReq.getComputingResourceCount(); i++){
			compRes = taskResReq.getComputingResource(i);
			explorer.explore(compRes);
			
			values = null;
			try {
				switch(param){
					case BANDWIDTH: 
							values = explorer.getRangeBandwidth();
						break;

					case CAPACITY: 
							values = explorer.getRangeCapacity();
						break;

					case LATENCY: 
							values = explorer.getRangeLatency();
						break;
				}
			} catch (NoSuchParamException e){
				values = null;
			}
			
			
			if(values != null && values.length > 0) {
				list.add(values[0]);
			} 
			
		}
		
		values = new double[list.size()][];
		for(int i = 0; i < list.size(); i++){
			values[i] = list.get(i);
		}
		
		return values;
	}

	
}
