package org.qcg.broker.schemas.resreqs.wrapper.impl;

import java.util.ArrayList;

import org.qcg.broker.schemas.exception.NoSuchParamException;
import org.qcg.broker.schemas.resreqs.ComputingResourceBaseTypeItem;
import org.qcg.broker.schemas.resreqs.ComputingResourceNetParameterType;
import org.qcg.broker.schemas.resreqs.ComputingResourceParameterType;
import org.qcg.broker.schemas.resreqs.ComputingResourceType;
import org.qcg.broker.schemas.resreqs.OtherParameterType;
import org.qcg.broker.schemas.resreqs.ParameterBoundaryType;
import org.qcg.broker.schemas.resreqs.ParameterTypeChoice;
import org.qcg.broker.schemas.resreqs.ParameterTypeChoiceItem;
import org.qcg.broker.schemas.resreqs.ParameterValue;
import org.qcg.broker.schemas.resreqs.Range;
import org.qcg.broker.schemas.resreqs.StringValue;
import org.qcg.broker.schemas.resreqs.types.ComputingResourceParameterTypeNameType;
import org.qcg.broker.schemas.resreqs.types.NetworkParameterTypeNameType;
import org.qcg.broker.schemas.resreqs.wrapper.ComputingResourceResReqParamExplorer;



/**
 * 
 * @author Marcin Krystek
 *
 */
public class ComputingResourceParamExplorerImpl implements ComputingResourceResReqParamExplorer {

	protected ComputingResourceType computingResource;
	
	public ComputingResourceParameterType getHostParam(ComputingResourceParameterTypeNameType paramName) throws NoSuchParamException{
		if(computingResource == null)
			throw new NoSuchParamException("Computing resource is unavailable. " +
											"Please set computing resource before exploring its values.");
		
		boolean found = false;
		ComputingResourceBaseTypeItem crItem = null;
		ComputingResourceParameterType hostParam = null;
		
		for(int i = 0; i < computingResource.getComputingResourceBaseTypeItemCount() && !found; i++){
			crItem = computingResource.getComputingResourceBaseTypeItem(i);
			if((hostParam = crItem.getHostParameter()) != null) {
				
				if(hostParam.getName().equals(paramName)){
					found = true;
				}
			}
		}
		
		if(!found)
			throw new NoSuchParamException(paramName, "Parameter is not defined.");
		
		return hostParam;
		
	}
	
	public ComputingResourceNetParameterType getNetParam(NetworkParameterTypeNameType paramName) throws NoSuchParamException {
		if(computingResource == null)
			throw new NoSuchParamException("Computing resource is unavailable. " +
											"Please set computing resource before exploring its values.");
		
		boolean found = false;
		ComputingResourceBaseTypeItem crItem = null;
		ComputingResourceNetParameterType netParam = null;
		
		for(int i = 0; i < computingResource.getComputingResourceBaseTypeItemCount() && !found; i++){
			crItem = computingResource.getComputingResourceBaseTypeItem(i);
			
			if((netParam = crItem.getNetworkParameter()) != null){
				
				if(netParam.getName().equals(paramName)){
					found = true;
				}
			}
		}
		
		if(!found)
			throw new NoSuchParamException(paramName, "Parameter is not defined.");
		
		return netParam;
	}
	
	public OtherParameterType getOtherParam(String paramName) throws NoSuchParamException{
		if(computingResource == null)
			throw new NoSuchParamException("Computing resource is unavailable. " +
											"Please set computing resource before exploring its values.");
		
		boolean found = false;
		ComputingResourceBaseTypeItem crItem = null;
		OtherParameterType otherParam = null;
		
		for(int i = 0; i < computingResource.getComputingResourceBaseTypeItemCount() && !found; i++){
			crItem = computingResource.getComputingResourceBaseTypeItem(i);
			
			if((otherParam = crItem.getOtherParameter()) != null) {
				
				if(otherParam.getName().equals(paramName)){
					found = true;
				}
			}
				
		}
		
		if(!found)
			throw new NoSuchParamException(paramName, "Parameter " + paramName + " is not defined.");
		
		return otherParam;
	}
	
	public String[] getApplicationName() throws NoSuchParamException {
		return getParamStringValue(ComputingResourceParameterTypeNameType.APPLICATION);
	}

	public String[] getCpuarch() throws NoSuchParamException {
		return getParamStringValue(ComputingResourceParameterTypeNameType.CPUARCH);
	}

	public double[] getCpucount() throws NoSuchParamException {
		return getParamDoubleValue(ComputingResourceParameterTypeNameType.CPUCOUNT);
	}

	public double[] getCpuLoad() throws NoSuchParamException {
		throw new NoSuchParamException("cpuload");
	}

	public double[] getCpuspeed() throws NoSuchParamException {
		return getParamDoubleValue(ComputingResourceParameterTypeNameType.CPUSPEED);
	}

	public double[] getDiskspace() throws NoSuchParamException {
		return getParamDoubleValue(ComputingResourceParameterTypeNameType.DISKSPACE);
	}

	public double[] getFreecpus() throws NoSuchParamException {
		return getParamDoubleValue(ComputingResourceParameterTypeNameType.FREECPUS);
	}

	public double[] getFreediskspace() throws NoSuchParamException {
		return getParamDoubleValue(ComputingResourceParameterTypeNameType.FREEDISKSPACE);
	}

	public double[] getFreememory() throws NoSuchParamException {
		return getParamDoubleValue(ComputingResourceParameterTypeNameType.FREEMEMORY);
	}

	public String[] getHostname() throws NoSuchParamException {
		return getParamStringValue(ComputingResourceParameterTypeNameType.HOSTNAME);
	}

	public String[] getLocalResourceManager() throws NoSuchParamException {
		return getParamStringValue(ComputingResourceParameterTypeNameType.LOCALRESOURCEMANAGER);
	}

	public double[] getMemory() throws NoSuchParamException {
		return getParamDoubleValue(ComputingResourceParameterTypeNameType.MEMORY);
	}

	public String[] getOsname() throws NoSuchParamException {
		return getParamStringValue(ComputingResourceParameterTypeNameType.OSNAME);
	}

	public String[] getOsrelease() throws NoSuchParamException {
		return getParamStringValue(ComputingResourceParameterTypeNameType.OSRELEASE);
	}

	public String[] getOstype() throws NoSuchParamException {
		return getParamStringValue(ComputingResourceParameterTypeNameType.OSTYPE);
	}

	public String[] getOsversion() throws NoSuchParamException {
		return getParamStringValue(ComputingResourceParameterTypeNameType.OSVERSION);
	}

	public String[] getRemoteSubmissionInterface() throws NoSuchParamException {
		return getParamStringValue(ComputingResourceParameterTypeNameType.REMOTESUBMISSIONINTERFACE);
	}
	public double[] getBandwidth() throws NoSuchParamException {
		return getParamDoubleValue(NetworkParameterTypeNameType.BANDWIDTH);
	}
	
	public double[] getLatency() throws NoSuchParamException {
		return getParamDoubleValue(NetworkParameterTypeNameType.LATENCY);
	}
		
	public double[] getCapacity() throws NoSuchParamException {
		return getParamDoubleValue(NetworkParameterTypeNameType.CAPACITY);
	}	

	public void explore(ComputingResourceType computingResource) {
		this.computingResource = computingResource;
	}

	public double getMaxBandwidth() throws NoSuchParamException {
		return getParamMaxValue(NetworkParameterTypeNameType.BANDWIDTH);
	}

	public double getMaxCapacity() throws NoSuchParamException {
		return getParamMaxValue(NetworkParameterTypeNameType.CAPACITY);
	}

	public double getMaxCpucount() throws NoSuchParamException {
		return getParamMaxValue(ComputingResourceParameterTypeNameType.CPUCOUNT);
	}

	public double getMaxCpuload() throws NoSuchParamException {
		throw new NoSuchParamException("cpuload");
	}

	public double getMaxCpuspeed() throws NoSuchParamException {
		return getParamMaxValue(ComputingResourceParameterTypeNameType.CPUSPEED);
	}

	public double getMaxDiskspace() throws NoSuchParamException {
		return getParamMaxValue(ComputingResourceParameterTypeNameType.DISKSPACE);
	}

	public double getMaxFreecpus() throws NoSuchParamException {
		return getParamMaxValue(ComputingResourceParameterTypeNameType.FREECPUS);
	}

	public double getMaxFreediskspace() throws NoSuchParamException {
		return getParamMaxValue(ComputingResourceParameterTypeNameType.FREEDISKSPACE);
	}

	public double getMaxFreememory() throws NoSuchParamException {
		return getParamMaxValue(ComputingResourceParameterTypeNameType.FREEMEMORY);
	}

	public double getMaxLatency() throws NoSuchParamException {
		return getParamMaxValue(NetworkParameterTypeNameType.LATENCY);
	}

	public double getMaxMemory() throws NoSuchParamException {
		return getParamMaxValue(ComputingResourceParameterTypeNameType.MEMORY);
	}

	public double getMinBandwidth() throws NoSuchParamException {
		return getParamMinValue(NetworkParameterTypeNameType.BANDWIDTH);
	}

	public double getMinCapacity() throws NoSuchParamException {
		return getParamMinValue(NetworkParameterTypeNameType.CAPACITY);
	}

	public double getMinCpucount() throws NoSuchParamException {
		return getParamMinValue(ComputingResourceParameterTypeNameType.CPUCOUNT);
	}

	public double getMinCpuload() throws NoSuchParamException {
		throw new NoSuchParamException("cpuload");
	}

	public double getMinCpuspeed() throws NoSuchParamException {
		return getParamMinValue(ComputingResourceParameterTypeNameType.CPUSPEED);
	}

	public double getMinDiskspace() throws NoSuchParamException {
		return getParamMinValue(ComputingResourceParameterTypeNameType.DISKSPACE);
	}

	public double getMinFreecpus() throws NoSuchParamException {
		return getParamMinValue(ComputingResourceParameterTypeNameType.FREECPUS);
	}

	public double getMinFreediskspace() throws NoSuchParamException {
		return getParamMinValue(ComputingResourceParameterTypeNameType.FREEDISKSPACE);
	}

	public double getMinFreememory() throws NoSuchParamException {
		return getParamMinValue(ComputingResourceParameterTypeNameType.FREEMEMORY);
	}

	public double getMinLatency() throws NoSuchParamException {
		return getParamMinValue(NetworkParameterTypeNameType.LATENCY);
	}

	public double getMinMemory() throws NoSuchParamException {
		return getParamMinValue(ComputingResourceParameterTypeNameType.MEMORY);
	}

	public double[][] getRangeBandwidth() throws NoSuchParamException {
		return getParamRangeValue(NetworkParameterTypeNameType.BANDWIDTH);
	}

	public double[][] getRangeCapacity() throws NoSuchParamException {
		return getParamRangeValue(NetworkParameterTypeNameType.CAPACITY);
	}

	public double[][] getRangeCpucount() throws NoSuchParamException {
		return getParamRangeValue(ComputingResourceParameterTypeNameType.CPUCOUNT);
	}

	public double[][] getRangeCpuload() throws NoSuchParamException {
		throw new NoSuchParamException("cpuload");
	}

	public double[][] getRangeCpuspeed() throws NoSuchParamException {
		return getParamRangeValue(ComputingResourceParameterTypeNameType.CPUSPEED);
	}

	public double[][] getRangeDiskspace() throws NoSuchParamException {
		return getParamRangeValue(ComputingResourceParameterTypeNameType.DISKSPACE);
	}

	public double[][] getRangeFreecpus() throws NoSuchParamException {
		return getParamRangeValue(ComputingResourceParameterTypeNameType.FREECPUS);
	}

	public double[][] getRangeFreediskspace() throws NoSuchParamException {
		return getParamRangeValue(ComputingResourceParameterTypeNameType.FREEDISKSPACE);
	}

	public double[][] getRangeFreememory() throws NoSuchParamException {
		return getParamRangeValue(ComputingResourceParameterTypeNameType.FREEMEMORY);
	}

	public double[][] getRangeLatency() throws NoSuchParamException {
		return getParamRangeValue(NetworkParameterTypeNameType.LATENCY);
	}

	public double[][] getRangeMemory() throws NoSuchParamException {
		return getParamRangeValue(ComputingResourceParameterTypeNameType.MEMORY);
	}
	
	
	
	protected double getParamMinValue(ComputingResourceParameterTypeNameType paramName) throws NoSuchParamException{
		ComputingResourceParameterType hostParam = null;
		ParameterBoundaryType pbt = null;
		
		hostParam = getHostParam(paramName);
		pbt = hostParam.getMin();
		if(pbt == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		return pbt.getContent();
	}
	
	
	protected double getParamMaxValue(ComputingResourceParameterTypeNameType paramName) throws NoSuchParamException{
		ComputingResourceParameterType hostParam = null;
		ParameterBoundaryType pbt = null;
		
		hostParam = getHostParam(paramName);
		pbt = hostParam.getMax();
		if(pbt == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		return pbt.getContent();
	}
	
	
	protected double[] getParamDoubleValue(ComputingResourceParameterTypeNameType paramName) throws NoSuchParamException{
		double value[] = null;
		ArrayList<Double> list = null;
		ComputingResourceParameterType hostParam = null;
		ParameterTypeChoice choice = null;
		ParameterTypeChoiceItem choiceItem = null;
		ParameterValue paramValue = null;
		
		hostParam = getHostParam(paramName);
		choice = hostParam.getParameterTypeChoice();

		if(choice == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");

		list = new ArrayList<Double>();
		for(int i = 0; i < choice.getParameterTypeChoiceItemCount(); i++){
			choiceItem = choice.getParameterTypeChoiceItem(i);
			paramValue = choiceItem.getParameterValue();
			if(paramValue == null) continue;
			
			list.add(paramValue.getContent());
			
		}
		
		value = new double[list.size()];

		for(int i = 0; i < value.length; i++){
			value[i] = list.get(i);
		}

		return value;
	}
	
	protected String[] getParamStringValue(ComputingResourceParameterTypeNameType paramName) throws NoSuchParamException{
		String value[] = null;
		StringValue stringValue[] = null;
		ComputingResourceParameterType hostParam = null;
		
		hostParam = getHostParam(paramName);
		stringValue = hostParam.getStringValue();
		if(stringValue == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		value = new String[stringValue.length];

		for(int i = 0; i < value.length; i++){
			value[i] = stringValue[i].getValue();
		}
		
		return value;
	}
	
	/**
	 * 
	 * @param paramName
	 * @return 2dim tale, tab[i][0] - min value, tab[i][1] - max value
	 * @throws NoSuchParamException
	 */
	protected double[][] getParamRangeValue(ComputingResourceParameterTypeNameType paramName) throws NoSuchParamException{
		double value[][] = null;
		ArrayList<Double> list = null;
		ComputingResourceParameterType hostParam = null;
		ParameterTypeChoice choice = null;
		ParameterTypeChoiceItem choiceItem = null;
		Range paramValue = null;
		
		hostParam = getHostParam(paramName);
		choice = hostParam.getParameterTypeChoice();

		if(choice == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");

		list = new ArrayList<Double>();
		for(int i = 0; i < choice.getParameterTypeChoiceItemCount(); i++){
			choiceItem = choice.getParameterTypeChoiceItem(i);
			paramValue = choiceItem.getRange();
			if(paramValue == null) continue;
			
			list.add(paramValue.getMin().getContent());
			list.add(paramValue.getMax().getContent());
			
		}
		
		value = new double[list.size() / 2][2];
		int j = 0;
		for(int i = 0; i < list.size(); i += 2, j++){
			value[j][0] = list.get(i);	 // min value
			value[j][1] = list.get(i+1); // max value
		}
		
		return value;
	}
	
	
	
	
	protected double getParamMinValue(NetworkParameterTypeNameType paramName) throws NoSuchParamException{
		ComputingResourceNetParameterType hostParam = null;
		ParameterBoundaryType pbt = null;
		
		hostParam = getNetParam(paramName);
		pbt = hostParam.getMin();
		if(pbt == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		return pbt.getContent();
	}
	
	
	protected double getParamMaxValue(NetworkParameterTypeNameType paramName) throws NoSuchParamException{
		ComputingResourceNetParameterType hostParam = null;
		ParameterBoundaryType pbt = null;
		
		hostParam = getNetParam(paramName);
		pbt = hostParam.getMax();
		if(pbt == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		return pbt.getContent();
	}
	
	
	protected double[] getParamDoubleValue(NetworkParameterTypeNameType paramName) throws NoSuchParamException{
		double value[] = null;
		ArrayList<Double> list = null;
		ComputingResourceNetParameterType hostParam = null;
		ParameterTypeChoice choice = null;
		ParameterTypeChoiceItem choiceItem = null;
		ParameterValue paramValue = null;
		
		hostParam = getNetParam(paramName);
		choice = hostParam.getParameterTypeChoice();

		if(choice == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");

		list = new ArrayList<Double>();
		for(int i = 0; i < choice.getParameterTypeChoiceItemCount(); i++){
			choiceItem = choice.getParameterTypeChoiceItem(i);
			paramValue = choiceItem.getParameterValue();
			if(paramValue == null) continue;
			
			list.add(paramValue.getContent());
			
		}
		
		value = new double[list.size()];

		for(int i = 0; i < value.length; i++){
			value[i] = list.get(i);
		}

		return value;
	}
	
	protected String[] getParamStringValue(NetworkParameterTypeNameType paramName) throws NoSuchParamException{
		String value[] = null;
		StringValue stringValue[] = null;
		ComputingResourceNetParameterType hostParam = null;
		
		hostParam = getNetParam(paramName);
		stringValue = hostParam.getStringValue();
		if(stringValue == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");
		
		value = new String[stringValue.length];

		for(int i = 0; i < value.length; i++){
			value[i] = stringValue[i].getValue();
		}
		
		return value;
	}
	
	/**
	 * 
	 * @param paramName
	 * @return 2dim tale, tab[i][0] - min value, tab[i][1] - max value
	 * @throws NoSuchParamException
	 */
	protected double[][] getParamRangeValue(NetworkParameterTypeNameType paramName) throws NoSuchParamException{
		double value[][] = null;
		ArrayList<Double> list = null;
		ComputingResourceNetParameterType hostParam = null;
		ParameterTypeChoice choice = null;
		ParameterTypeChoiceItem choiceItem = null;
		Range paramValue = null;
		
		hostParam = getNetParam(paramName);
		choice = hostParam.getParameterTypeChoice();

		if(choice == null)
			throw new NoSuchParamException("Value for "+paramName.value() + " parameter is not defined.");

		list = new ArrayList<Double>();
		for(int i = 0; i < choice.getParameterTypeChoiceItemCount(); i++){
			choiceItem = choice.getParameterTypeChoiceItem(i);
			paramValue = choiceItem.getRange();
			if(paramValue == null) continue;
			
			list.add(paramValue.getMin().getContent());
			list.add(paramValue.getMax().getContent());
			
		}
		
		value = new double[list.size() / 2][2];
		int j = 0;
		for(int i = 0; i < list.size(); i += 2, j++){
			value[j][0] = list.get(i);	 // min value
			value[j][1] = list.get(i+1); // max value
		}
		
		return value;
	}

}
