package org.qcg.broker.schemas.utils;

import org.qcg.broker.schemas.exception.NoSuchParamException;


/**
 * 
 * @author Marcin Krystek
 *
 */
public interface ComputingResourceParamExplorer<ComputingResource, ComputingResourceNetParameterType, 
												ComputingResourceParameterType, OtherParameterType, 
												ComputingParameterName, NetworkParameterName> {
	
	/**
	 * Use this method to make ComputingResource object available for exploration.
	 * @param computingResource object that will be explored.
	 */
	public void explore(ComputingResource computingResource);

	/**
	 * 
	 * @return value of ComputingParameterName.OSNAME parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getOsname() throws NoSuchParamException;

	/**
	 * 
	 * @return value of ComputingParameterName.OSTYPE parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getOstype() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.CPUARCH parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getCpuarch() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.OSVERSION parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getOsversion() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.RELEASE parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getOsrelease() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.MEMORY parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getMemory() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.FREEMEMORY parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getFreememory() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.CPUCOUNT parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getCpucount() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.FREECPUS parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getFreecpus() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.CPULOAD parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getCpuLoad() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.CPUSPEED parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getCpuspeed() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.APPLICATION parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getApplicationName() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.DISKSPACE parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getDiskspace() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.FREEDISKSPACE parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getFreediskspace() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.REMOTESUBMISSIONINTERFACE parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getRemoteSubmissionInterface() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.LOCALRESOURCEMANAGER parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getLocalResourceManager() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of ComputingParameterName.HOSTNAME parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public String[] getHostname() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of NetworkParameterName.BANDWIDTH parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getBandwidth() throws NoSuchParamException;

	/**
	 * 
	 * @return value of NetworkParameterName.LATENCY parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getLatency() throws NoSuchParamException;
	
	/**
	 * 
	 * @return value of NetworkParameterName.CAPACITY parameter.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public double[] getCapacity() throws NoSuchParamException;
	
	/**
	 * @param paramName name of the requested parameter.
	 * @return parameter object. This is the object representation of the "hostParameter" element in schema.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public ComputingResourceParameterType getHostParam(ComputingParameterName paramName) throws NoSuchParamException;

	/**
	 * @param paramName name of the requested parameter.
	 * @return parameter object. This is the object representation of the "networkParameter" element in schema.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public ComputingResourceNetParameterType getNetParam(NetworkParameterName paramName) throws NoSuchParamException;
	
	/**
	 * @param paramName parameter name.
	 * @return parameter object. This is the object representation of the "otherParameter" element in schema.
	 * @throws NoSuchParamException if parameter is not defined.
	 */
	public OtherParameterType getOtherParam(String paramName) throws NoSuchParamException;
	
}
